import pickle


# Technion courses class
# contains information about an course
# e.g:
#   name   = "מערכות הפעלה"
#   number = "234123"
#   dependencies = {[234218]} courses which are pre-requisite
#   parallel = {[234118]} courses which are either pre-requisite or must be taken together with the course
#   similarities = {[]} courses which are similar to this course
#   inclusive = {[]} courses which are included in this course and covered by it
class Course:
    def __init__(self):
        self.name = ""
        self.number = ""
        self.points = 0
        self.dependencies = list()
        self.parallel = set()
        self.similarities = set()
        self.inclusive = set()
        self.english = False

    def set_name(self, name):
        self.name = name

    def set_number(self, number):
        self.number = number

    def set_points(self, points):
        try:
            self.points = float(points)
        except ValueError:
            self.points = 0

    def add_inclusive(self, courses):
        self.inclusive.update(courses)

    def add_dependencies(self, courses):
        self.dependencies.extend(courses)

    def add_similarities(self, courses):
        self.similarities.update(courses)

    def add_parallel(self, courses):
        self.parallel.update(courses)

    # Prepare the course data into a list to insert into DB (some is getting serialized)
    def to_list(self):
        return [
            self.name, self.number, self.points,
            pickle.dumps(self.dependencies),
            pickle.dumps(self.parallel),
            pickle.dumps(self.similarities),
            pickle.dumps(self.inclusive)
        ]

    def reprDependencies(self, english=False):
        if len(self.dependencies) > 0:
            result = []
            if english:
                humanify = str.maketrans({",": " and", "'": None})
            else:
                humanify = str.maketrans({",": " ו-", "'": None})
            separator = ""
            for depend in self.dependencies:
                result.append(separator)
                result.append(str(depend).translate(humanify))
                if english:
                    separator = " or "
                else:
                    separator = " או- "
            return ''.join(result)
        else:
            return ""

    def repOtherData(self, data):
        if len(data) > 0:
            result = ""
            humanify = str.maketrans({"{": None, "}": None, "'": None})
            return str(data).translate(humanify)
        else:
            return ""

    def __repr__(self):
        if self.english:
            represent = "Course name: {} \n".format(self.name) \
                        + "Course number: {} \n".format(self.number) \
                        + ("Points: {} \n".format(self.points) if self.points > 0 else "") \
                        + ("Pre-requisites: {} \n".format(self.reprDependencies(english=True)) if len(
                self.dependencies) > 0 else "") \
                        + ("Parallel courses: {} \n".format(self.repOtherData(self.parallel)) if len(
                self.parallel) > 0 else "") \
                        + ("Similar courses: {} \n".format(self.repOtherData(self.similarities)) if len(
                self.similarities) > 0 else "") \
                        + ("Inclusive courses: {} \n".format(self.repOtherData(self.inclusive)) if len(
                self.inclusive) > 0 else "")
        else:
            represent = "שם הקורס: {} \n".format(self.name) \
                        + "מספר קורס: {} \n".format(self.number) \
                        + ("מס' נקודות: {} \n".format(self.points) if self.points > 0 else "") \
                        + ("מקצועות קדם: {} \n".format(self.reprDependencies()) if len(self.dependencies) > 0 else "") \
                        + ("מקצועות צמודים: {} \n".format(self.repOtherData(self.parallel)) if len(
                self.parallel) > 0 else "") \
                        + ("מקצועות ללא זיכוי נוסף: {} \n".format(self.repOtherData(self.similarities)) if len(
                self.similarities) > 0 else "") \
                        + ("מקצועות ללא זיכוי נוסף (מוכלים): {} \n".format(self.repOtherData(self.inclusive)) if len(
                self.inclusive) > 0 else "")
        return represent



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'heb_tab.ui'
#
# Created by: PyQt5 UI code generator 5.12.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class TabPage(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(1064, 541)
        Form.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.semester_average_label = QtWidgets.QLabel(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.semester_average_label.sizePolicy().hasHeightForWidth())
        self.semester_average_label.setSizePolicy(sizePolicy)
        self.semester_average_label.setMaximumSize(QtCore.QSize(70, 16777215))
        self.semester_average_label.setObjectName("semester_average_label")
        self.horizontalLayout_7.addWidget(self.semester_average_label)
        self.semester_average_in = QtWidgets.QLineEdit(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.semester_average_in.sizePolicy().hasHeightForWidth())
        self.semester_average_in.setSizePolicy(sizePolicy)
        self.semester_average_in.setMaximumSize(QtCore.QSize(75, 16777215))
        self.semester_average_in.setText("")
        self.semester_average_in.setAlignment(QtCore.Qt.AlignCenter)
        self.semester_average_in.setReadOnly(True)
        self.semester_average_in.setObjectName("semester_average_in")
        self.horizontalLayout_7.addWidget(self.semester_average_in)
        self.verticalLayout_4.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.semester_points_label = QtWidgets.QLabel(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.semester_points_label.sizePolicy().hasHeightForWidth())
        self.semester_points_label.setSizePolicy(sizePolicy)
        self.semester_points_label.setMaximumSize(QtCore.QSize(70, 16777215))
        self.semester_points_label.setObjectName("semester_points_label")
        self.horizontalLayout_8.addWidget(self.semester_points_label)
        self.semester_points_in = QtWidgets.QLineEdit(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.semester_points_in.sizePolicy().hasHeightForWidth())
        self.semester_points_in.setSizePolicy(sizePolicy)
        self.semester_points_in.setMaximumSize(QtCore.QSize(75, 16777215))
        self.semester_points_in.setText("")
        self.semester_points_in.setAlignment(QtCore.Qt.AlignCenter)
        self.semester_points_in.setReadOnly(True)
        self.semester_points_in.setObjectName("semester_points_in")
        self.horizontalLayout_8.addWidget(self.semester_points_in)
        self.verticalLayout_4.addLayout(self.horizontalLayout_8)
        self.verticalLayout.addLayout(self.verticalLayout_4)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.gridLayout.addLayout(self.horizontalLayout, 0, 1, 1, 1)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem1)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem2)
        self.semester_table_add_line = QtWidgets.QPushButton(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.semester_table_add_line.sizePolicy().hasHeightForWidth())
        self.semester_table_add_line.setSizePolicy(sizePolicy)
        self.semester_table_add_line.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.semester_table_add_line.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("images/list-add.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.semester_table_add_line.setIcon(icon)
        self.semester_table_add_line.setIconSize(QtCore.QSize(32, 32))
        self.semester_table_add_line.setObjectName("semester_table_add_line")
        self.horizontalLayout_6.addWidget(self.semester_table_add_line)
        self.semester_table_remove_line = QtWidgets.QPushButton(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.semester_table_remove_line.sizePolicy().hasHeightForWidth())
        self.semester_table_remove_line.setSizePolicy(sizePolicy)
        self.semester_table_remove_line.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.semester_table_remove_line.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap("images/list-remove.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.semester_table_remove_line.setIcon(icon1)
        self.semester_table_remove_line.setIconSize(QtCore.QSize(32, 32))
        self.semester_table_remove_line.setObjectName("semester_table_remove_line")
        self.horizontalLayout_6.addWidget(self.semester_table_remove_line)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem3)
        self.gridLayout.addLayout(self.horizontalLayout_6, 1, 0, 1, 1)
        self.courses_table = QtWidgets.QTableWidget(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.courses_table.sizePolicy().hasHeightForWidth())
        self.courses_table.setSizePolicy(sizePolicy)
        self.courses_table.setMouseTracking(True)
        self.courses_table.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.courses_table.setObjectName("courses_table")
        self.courses_table.setColumnCount(6)
        self.courses_table.setRowCount(5)
        item = QtWidgets.QTableWidgetItem()
        self.courses_table.setVerticalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.courses_table.setVerticalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.courses_table.setVerticalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.courses_table.setVerticalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.courses_table.setVerticalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.courses_table.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.courses_table.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.courses_table.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.courses_table.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.courses_table.setHorizontalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.courses_table.setHorizontalHeaderItem(5, item)
        self.courses_table.horizontalHeader().setVisible(True)
        self.courses_table.horizontalHeader().setCascadingSectionResizes(False)
        self.courses_table.horizontalHeader().setSortIndicatorShown(True)
        self.courses_table.horizontalHeader().setStretchLastSection(False)
        self.courses_table.verticalHeader().setVisible(False)
        self.courses_table.verticalHeader().setCascadingSectionResizes(True)
        self.courses_table.verticalHeader().setStretchLastSection(False)
        self.gridLayout.addWidget(self.courses_table, 0, 0, 1, 1)
        self.semester_add_course = QtWidgets.QPushButton(Form)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(False)
        font.setItalic(False)
        font.setWeight(50)
        self.semester_add_course.setFont(font)
        self.semester_add_course.setObjectName("semester_add_course")
        self.gridLayout.addWidget(self.semester_add_course, 2, 0, 1, 1)
        self.courses_table.setSortingEnabled(True)
        self.not_show = False
        self.courses_table.setMouseTracking(False)
        self.courses_table.setSortingEnabled(False)
        for row in range(0, self.courses_table.rowCount()):
            item1 = QtWidgets.QTableWidgetItem()
            item2 = QtWidgets.QTableWidgetItem()
            self.courses_table.setItem(row, 0, item1)
            self.courses_table.setCellWidget(row, 0, createComboBox())
            self.courses_table.setItem(row, self.courses_table.columnCount() - 1, item2)
            self.courses_table.setCellWidget(row, self.courses_table.columnCount() - 1,
                                             createRemoveLineButton(str(row)))
            for column in range(1, self.courses_table.columnCount() - 1):
                item = QtWidgets.QTableWidgetItem()
                item.setTextAlignment(QtCore.Qt.AlignCenter)
                self.courses_table.setItem(row, column, item)
                if column >= 3:
                    if column == 3:
                        spin_box = QtWidgets.QDoubleSpinBox()
                        spin_box.setDecimals(1)
                        spin_box.setSingleStep(0.5)
                    else:
                        spin_box = QtWidgets.QSpinBox()
                        spin_box.setSingleStep(1)
                    spin_box.setRange(0, 100)
                    spin_box.setAlignment(QtCore.Qt.AlignCenter)
                    spin_box.setButtonSymbols(QtWidgets.QAbstractSpinBox.NoButtons)
                    self.courses_table.setCellWidget(row, column, spin_box)
        self.courses_table.horizontalHeader().setSectionResizeMode(
            QtWidgets.QHeaderView.Stretch)
        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.semester_average_label.setText(_translate("Form", "ממוצע:   "))
        self.semester_points_label.setText(_translate("Form", "מס\' נק\"ז:"))
        self.semester_table_add_line.setToolTip(_translate("Form",
                                                           "<html><head/><body><p><span style=\" font-size:12pt; font-weight:600;\">הוסף שורה</span></p></body></html>"))
        self.semester_table_remove_line.setToolTip(_translate("Form",
                                                              "<html><head/><body><p><span style=\" font-size:12pt; font-weight:600;\">הסר שורה</span></p></body></html>"))
        self.courses_table.setSortingEnabled(True)
        item = self.courses_table.verticalHeaderItem(0)
        item.setText(_translate("Form", "New Row"))
        item = self.courses_table.verticalHeaderItem(1)
        item.setText(_translate("Form", "New Row"))
        item = self.courses_table.verticalHeaderItem(2)
        item.setText(_translate("Form", "New Row"))
        item = self.courses_table.verticalHeaderItem(3)
        item.setText(_translate("Form", "New Row"))
        item = self.courses_table.verticalHeaderItem(4)
        item.setText(_translate("Form", "New Row"))
        item = self.courses_table.horizontalHeaderItem(0)
        item.setText(_translate("Form", "קטגוריה"))
        item = self.courses_table.horizontalHeaderItem(1)
        item.setText(_translate("Form", "מספר קורס"))
        item = self.courses_table.horizontalHeaderItem(2)
        item.setText(_translate("Form", "שם קורס"))
        item = self.courses_table.horizontalHeaderItem(3)
        item.setText(_translate("Form", "נק\"ז"))
        item = self.courses_table.horizontalHeaderItem(4)
        item.setText(_translate("Form", "ציון"))
        self.semester_add_course.setText(_translate("Form", "חפש קורס"))


def createComboBox():
    combo_box = QtWidgets.QComboBox()
    combo_box.setFocusPolicy(QtCore.Qt.StrongFocus)
    combo_box.setLayoutDirection(QtCore.Qt.RightToLeft)
    combo_box.addItem("חובה")
    combo_box.addItem("רשימה א")
    combo_box.addItem("רשימה ב")
    combo_box.addItem("פרוייקט")
    combo_box.addItem("מל\"ג")
    combo_box.addItem("ספורט")
    combo_box.addItem("חופשי")
    return combo_box


def createRemoveLineButton(line):
    semester_table_remove_line = QtWidgets.QPushButton()
    sizePolicy = QtWidgets.QSizePolicy(
        QtWidgets.QSizePolicy.MinimumExpanding,
        QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(
        semester_table_remove_line.sizePolicy().hasHeightForWidth())
    semester_table_remove_line.setSizePolicy(sizePolicy)
    semester_table_remove_line.setLayoutDirection(
        QtCore.Qt.LeftToRight)
    semester_table_remove_line.setText("")
    icon1 = QtGui.QIcon()
    icon1.addPixmap(QtGui.QPixmap("images/row-clean.svg"),
                    QtGui.QIcon.Normal, QtGui.QIcon.Off)
    semester_table_remove_line.setIcon(icon1)
    semester_table_remove_line.setIconSize(QtCore.QSize(32, 32))
    semester_table_remove_line.setObjectName(
        line)
    _translate = QtCore.QCoreApplication.translate
    semester_table_remove_line.setToolTip(
        _translate(
            "Form",
            "<html><head/><body><p><span style=\" font-size:12pt; font-weight:600;\">איפוס שורה</span></p></body></html>"
        ))
    return semester_table_remove_line



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'english_design.ui'
#
# Created by: PyQt5 UI code generator 5.12.1
#
# WARNING! All changes made in this file will be lost!

import json

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        width, height = loadDimensions()
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(width, height)
        MainWindow.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.horizontalLayout_22 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_22.setObjectName("horizontalLayout_22")
        self.semesters = list()
        self.verticalLayout_15 = QtWidgets.QVBoxLayout()
        self.verticalLayout_15.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.verticalLayout_15.setContentsMargins(-1, 0, 0, -1)
        self.verticalLayout_15.setSpacing(0)
        self.verticalLayout_15.setObjectName("verticalLayout_15")
        self.progress_label_8 = QtWidgets.QLabel(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.progress_label_8.sizePolicy().hasHeightForWidth())
        self.progress_label_8.setSizePolicy(sizePolicy)
        self.progress_label_8.setMaximumSize(QtCore.QSize(600, 16777215))
        font = QtGui.QFont()
        font.setFamily("Noto Sans")
        font.setPointSize(13)
        font.setBold(False)
        font.setItalic(False)
        font.setUnderline(True)
        font.setWeight(50)
        self.progress_label_8.setFont(font)
        self.progress_label_8.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.progress_label_8.setStyleSheet("font: 13pt \"Noto Sans\";\n"
                                            "text-decoration: underline;")
        self.progress_label_8.setAlignment(QtCore.Qt.AlignBottom | QtCore.Qt.AlignLeading | QtCore.Qt.AlignLeft)
        self.progress_label_8.setObjectName("progress_label_8")
        self.verticalLayout_15.addWidget(self.progress_label_8)
        self.frame_11 = QtWidgets.QFrame(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame_11.sizePolicy().hasHeightForWidth())
        self.frame_11.setSizePolicy(sizePolicy)
        self.frame_11.setMaximumSize(QtCore.QSize(600, 16777215))
        self.frame_11.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_11.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_11.setObjectName("frame_11")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.frame_11)
        self.gridLayout_10.setContentsMargins(0, 2, 0, 4)
        self.gridLayout_10.setHorizontalSpacing(0)
        self.gridLayout_10.setVerticalSpacing(3)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.horizontalLayout_44 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_44.setContentsMargins(0, -1, -1, -1)
        self.horizontalLayout_44.setSpacing(0)
        self.horizontalLayout_44.setObjectName("horizontalLayout_44")
        self.frame = QtWidgets.QFrame(self.frame_11)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy)
        self.frame.setMaximumSize(QtCore.QSize(200, 16777215))
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.frame)
        self.gridLayout_7.setContentsMargins(0, -1, 2, -1)
        self.gridLayout_7.setHorizontalSpacing(2)
        self.gridLayout_7.setVerticalSpacing(6)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.verticalLayout_21 = QtWidgets.QVBoxLayout()
        self.verticalLayout_21.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.verticalLayout_21.setContentsMargins(6, -1, 6, -1)
        self.verticalLayout_21.setSpacing(3)
        self.verticalLayout_21.setObjectName("verticalLayout_21")
        self.desc_label = QtWidgets.QLabel(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.desc_label.sizePolicy().hasHeightForWidth())
        self.desc_label.setSizePolicy(sizePolicy)
        self.desc_label.setMaximumSize(QtCore.QSize(110, 16777215))
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setUnderline(True)
        self.desc_label.setFont(font)
        self.desc_label.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.desc_label.setScaledContents(False)
        self.desc_label.setObjectName("desc_label")
        self.verticalLayout_21.addWidget(self.desc_label)
        self.horizontalLayout_80 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_80.setSpacing(12)
        self.horizontalLayout_80.setObjectName("horizontalLayout_80")
        self.average_in_7 = QtWidgets.QLineEdit(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.average_in_7.sizePolicy().hasHeightForWidth())
        self.average_in_7.setSizePolicy(sizePolicy)
        self.average_in_7.setMaximumSize(QtCore.QSize(60, 16777215))
        self.average_in_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.average_in_7.setText("")
        self.average_in_7.setAlignment(QtCore.Qt.AlignCenter)
        self.average_in_7.setReadOnly(True)
        self.average_in_7.setObjectName("average_in_7")
        self.horizontalLayout_80.addWidget(self.average_in_7)
        self.average_lab_7 = QtWidgets.QLabel(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.average_lab_7.sizePolicy().hasHeightForWidth())
        self.average_lab_7.setSizePolicy(sizePolicy)
        self.average_lab_7.setMaximumSize(QtCore.QSize(110, 16777215))
        font = QtGui.QFont()
        # font.setUnderline(True)
        self.average_lab_7.setFont(font)
        self.average_lab_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.average_lab_7.setObjectName("average_lab_7")
        self.horizontalLayout_80.addWidget(self.average_lab_7)
        self.verticalLayout_21.addLayout(self.horizontalLayout_80)
        self.horizontalLayout_81 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_81.setSpacing(12)
        self.horizontalLayout_81.setObjectName("horizontalLayout_81")
        self.points_in_7 = QtWidgets.QLineEdit(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.points_in_7.sizePolicy().hasHeightForWidth())
        self.points_in_7.setSizePolicy(sizePolicy)
        self.points_in_7.setMaximumSize(QtCore.QSize(60, 16777215))
        self.points_in_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.points_in_7.setText("")
        self.points_in_7.setAlignment(QtCore.Qt.AlignCenter)
        self.points_in_7.setReadOnly(True)
        self.points_in_7.setObjectName("points_in_7")
        self.horizontalLayout_81.addWidget(self.points_in_7)
        self.points_label_7 = QtWidgets.QLabel(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.points_label_7.sizePolicy().hasHeightForWidth())
        self.points_label_7.setSizePolicy(sizePolicy)
        self.points_label_7.setMaximumSize(QtCore.QSize(110, 16777215))
        font = QtGui.QFont()
        # font.setUnderline(True)
        self.points_label_7.setFont(font)
        self.points_label_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.points_label_7.setObjectName("points_label_7")
        self.horizontalLayout_81.addWidget(self.points_label_7)
        self.verticalLayout_21.addLayout(self.horizontalLayout_81)
        self.horizontalLayout_82 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_82.setSpacing(12)
        self.horizontalLayout_82.setObjectName("horizontalLayout_82")
        self.points_left_in_7 = QtWidgets.QLineEdit(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.points_left_in_7.sizePolicy().hasHeightForWidth())
        self.points_left_in_7.setSizePolicy(sizePolicy)
        self.points_left_in_7.setMaximumSize(QtCore.QSize(60, 16777215))
        self.points_left_in_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.points_left_in_7.setText("")
        self.points_left_in_7.setAlignment(QtCore.Qt.AlignCenter)
        self.points_left_in_7.setReadOnly(True)
        self.points_left_in_7.setObjectName("points_left_in_7")
        self.horizontalLayout_82.addWidget(self.points_left_in_7)
        self.points_left_label_7 = QtWidgets.QLabel(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.points_left_label_7.sizePolicy().hasHeightForWidth())
        self.points_left_label_7.setSizePolicy(sizePolicy)
        self.points_left_label_7.setMaximumSize(QtCore.QSize(110, 16777215))
        font = QtGui.QFont()
        # font.setUnderline(True)
        self.points_left_label_7.setFont(font)
        self.points_left_label_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.points_left_label_7.setObjectName("points_left_label_7")
        self.horizontalLayout_82.addWidget(self.points_left_label_7)
        self.verticalLayout_21.addLayout(self.horizontalLayout_82)
        self.horizontalLayout_83 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_83.setSpacing(12)
        self.horizontalLayout_83.setObjectName("horizontalLayout_83")
        self.points_left_to_choose_in_7 = QtWidgets.QLineEdit(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.points_left_to_choose_in_7.sizePolicy().hasHeightForWidth())
        self.points_left_to_choose_in_7.setSizePolicy(sizePolicy)
        self.points_left_to_choose_in_7.setMaximumSize(QtCore.QSize(60, 16777215))
        self.points_left_to_choose_in_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.points_left_to_choose_in_7.setText("")
        self.points_left_to_choose_in_7.setAlignment(QtCore.Qt.AlignCenter)
        self.points_left_to_choose_in_7.setReadOnly(True)
        self.points_left_to_choose_in_7.setObjectName("points_left_to_choose_in_7")
        self.horizontalLayout_83.addWidget(self.points_left_to_choose_in_7)
        self.points_left_to_choose_label_7 = QtWidgets.QLabel(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.points_left_to_choose_label_7.sizePolicy().hasHeightForWidth())
        self.points_left_to_choose_label_7.setSizePolicy(sizePolicy)
        self.points_left_to_choose_label_7.setMaximumSize(QtCore.QSize(110, 16777215))
        font = QtGui.QFont()
        # font.setUnderline(True)
        self.points_left_to_choose_label_7.setFont(font)
        self.points_left_to_choose_label_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.points_left_to_choose_label_7.setObjectName("points_left_to_choose_label_7")
        self.horizontalLayout_83.addWidget(self.points_left_to_choose_label_7)
        self.verticalLayout_21.addLayout(self.horizontalLayout_83)
        spacerItem = QtWidgets.QSpacerItem(20, 77, QtWidgets.QSizePolicy.Minimum,
                                           QtWidgets.QSizePolicy.MinimumExpanding)
        self.verticalLayout_21.addItem(spacerItem)
        self.gridLayout_7.addLayout(self.verticalLayout_21, 0, 0, 1, 1)
        self.frame_3 = QtWidgets.QFrame(self.frame_11)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame_3.sizePolicy().hasHeightForWidth())
        self.frame_3.setSizePolicy(sizePolicy)
        self.frame_3.setMaximumSize(QtCore.QSize(340, 16777215))
        self.frame_3.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_3.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_3.setObjectName("frame_3")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.frame_3)
        self.gridLayout_8.setContentsMargins(2, -1, 2, -1)
        self.gridLayout_8.setHorizontalSpacing(2)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.verticalLayout_23 = QtWidgets.QVBoxLayout()
        self.verticalLayout_23.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.verticalLayout_23.setSpacing(5)
        self.verticalLayout_23.setObjectName("verticalLayout_23")
        self.horizontalLayout_85 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_85.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.horizontalLayout_85.setSpacing(12)
        self.horizontalLayout_85.setObjectName("horizontalLayout_85")
        spacerItem1 = QtWidgets.QSpacerItem(50, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.done_label_7 = QtWidgets.QLabel(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.done_label_7.sizePolicy().hasHeightForWidth())
        self.done_label_7.setSizePolicy(sizePolicy)
        self.done_label_7.setMaximumSize(QtCore.QSize(110, 16777215))
        font = QtGui.QFont()
        font.setFamily("Noto Sans")
        font.setPointSize(10)
        font.setBold(False)
        font.setItalic(False)
        font.setUnderline(True)
        font.setWeight(7)
        self.done_label_7.setFont(font)
        self.done_label_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.done_label_7.setStyleSheet("font: 57 10pt \"Noto Sans\";")
        self.done_label_7.setObjectName("done_label_7")
        self.of_label_7 = QtWidgets.QLabel(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.of_label_7.sizePolicy().hasHeightForWidth())
        self.of_label_7.setSizePolicy(sizePolicy)
        self.of_label_7.setMaximumSize(QtCore.QSize(110, 16777215))
        font = QtGui.QFont()
        font.setFamily("Noto Sans")
        font.setPointSize(10)
        font.setBold(False)
        font.setItalic(False)
        font.setUnderline(True)
        font.setWeight(7)
        self.of_label_7.setFont(font)
        self.of_label_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.of_label_7.setStyleSheet("font: 57 10pt \"Noto Sans\";")
        self.of_label_7.setObjectName("of_label_7")
        self.horizontalLayout_85.addWidget(self.of_label_7)
        self.horizontalLayout_85.addItem(spacerItem1)
        self.horizontalLayout_85.addWidget(self.done_label_7)
        spacerItem2 = QtWidgets.QSpacerItem(150, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_85.addItem(spacerItem2)
        self.verticalLayout_23.addLayout(self.horizontalLayout_85)
        self.horizontalLayout_92 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_92.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.horizontalLayout_92.setSpacing(12)
        self.horizontalLayout_92.setObjectName("horizontalLayout_92")
        self.must_of_in = QtWidgets.QDoubleSpinBox(self.frame_3)
        self.must_of_in.setRange(0, 1000)
        self.must_of_in.setDecimals(1)
        self.must_of_in.setSingleStep(0.5)
        self.must_of_in.setAlignment(QtCore.Qt.AlignCenter)
        self.must_of_in.setButtonSymbols(QtWidgets.QAbstractSpinBox.NoButtons)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.must_of_in.sizePolicy().hasHeightForWidth())
        self.must_of_in.setSizePolicy(sizePolicy)
        self.must_of_in.setMaximumSize(QtCore.QSize(75, 16777215))
        self.must_of_in.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.must_of_in.setAlignment(QtCore.Qt.AlignCenter)
        self.must_of_in.setObjectName("must_of_in")
        self.horizontalLayout_92.addWidget(self.must_of_in)
        self.must_done_in = QtWidgets.QLineEdit(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.must_done_in.sizePolicy().hasHeightForWidth())
        self.must_done_in.setSizePolicy(sizePolicy)
        self.must_done_in.setMaximumSize(QtCore.QSize(75, 16777215))
        self.must_done_in.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.must_done_in.setText("")
        self.must_done_in.setAlignment(QtCore.Qt.AlignCenter)
        self.must_done_in.setReadOnly(True)
        self.must_done_in.setObjectName("must_done_in")
        self.horizontalLayout_92.addWidget(self.must_done_in)
        self.must_label = QtWidgets.QLabel(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.must_label.sizePolicy().hasHeightForWidth())
        self.must_label.setSizePolicy(sizePolicy)
        self.must_label.setMaximumSize(QtCore.QSize(110, 16777215))
        font = QtGui.QFont()
        # font.setUnderline(True)
        self.must_label.setFont(font)
        self.must_label.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.must_label.setObjectName("must_label")
        self.horizontalLayout_92.addWidget(self.must_label)
        self.verticalLayout_23.addLayout(self.horizontalLayout_92)
        self.horizontalLayout_86 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_86.setSpacing(12)
        self.horizontalLayout_86.setObjectName("horizontalLayout_86")
        self.list_a_of_in_7 = QtWidgets.QDoubleSpinBox(self.frame_3)
        self.list_a_of_in_7.setRange(0, 1000)
        self.list_a_of_in_7.setDecimals(1)
        self.list_a_of_in_7.setSingleStep(0.5)
        self.list_a_of_in_7.setAlignment(QtCore.Qt.AlignCenter)
        self.list_a_of_in_7.setButtonSymbols(QtWidgets.QAbstractSpinBox.NoButtons)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.list_a_of_in_7.sizePolicy().hasHeightForWidth())
        self.list_a_of_in_7.setSizePolicy(sizePolicy)
        self.list_a_of_in_7.setMaximumSize(QtCore.QSize(75, 16777215))
        self.list_a_of_in_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.list_a_of_in_7.setAlignment(QtCore.Qt.AlignCenter)
        self.list_a_of_in_7.setObjectName("list_a_of_in_7")
        self.horizontalLayout_86.addWidget(self.list_a_of_in_7)
        self.list_a_done_in_7 = QtWidgets.QLineEdit(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.list_a_done_in_7.sizePolicy().hasHeightForWidth())
        self.list_a_done_in_7.setSizePolicy(sizePolicy)
        self.list_a_done_in_7.setMaximumSize(QtCore.QSize(75, 16777215))
        self.list_a_done_in_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.list_a_done_in_7.setText("")
        self.list_a_done_in_7.setAlignment(QtCore.Qt.AlignCenter)
        self.list_a_done_in_7.setReadOnly(True)
        self.list_a_done_in_7.setObjectName("list_a_done_in_7")
        self.horizontalLayout_86.addWidget(self.list_a_done_in_7)
        self.list_a_label_7 = QtWidgets.QLabel(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.list_a_label_7.sizePolicy().hasHeightForWidth())
        self.list_a_label_7.setSizePolicy(sizePolicy)
        self.list_a_label_7.setMaximumSize(QtCore.QSize(110, 16777215))
        font = QtGui.QFont()
        # font.setUnderline(True)
        self.list_a_label_7.setFont(font)
        self.list_a_label_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.list_a_label_7.setObjectName("list_a_label_7")
        self.horizontalLayout_86.addWidget(self.list_a_label_7)
        self.verticalLayout_23.addLayout(self.horizontalLayout_86)
        self.horizontalLayout_87 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_87.setSpacing(12)
        self.horizontalLayout_87.setObjectName("horizontalLayout_87")
        self.list_b_of_in_7 = QtWidgets.QDoubleSpinBox(self.frame_3)
        self.list_b_of_in_7.setRange(0, 1000)
        self.list_b_of_in_7.setDecimals(1)
        self.list_b_of_in_7.setSingleStep(0.5)
        self.list_b_of_in_7.setAlignment(QtCore.Qt.AlignCenter)
        self.list_b_of_in_7.setButtonSymbols(QtWidgets.QAbstractSpinBox.NoButtons)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.list_b_of_in_7.sizePolicy().hasHeightForWidth())
        self.list_b_of_in_7.setSizePolicy(sizePolicy)
        self.list_b_of_in_7.setMaximumSize(QtCore.QSize(75, 16777215))
        self.list_b_of_in_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.list_b_of_in_7.setAlignment(QtCore.Qt.AlignCenter)
        self.list_b_of_in_7.setObjectName("list_b_of_in_7")
        self.horizontalLayout_87.addWidget(self.list_b_of_in_7)
        self.list_b_done_in_7 = QtWidgets.QLineEdit(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.list_b_done_in_7.sizePolicy().hasHeightForWidth())
        self.list_b_done_in_7.setSizePolicy(sizePolicy)
        self.list_b_done_in_7.setMaximumSize(QtCore.QSize(75, 16777215))
        self.list_b_done_in_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.list_b_done_in_7.setText("")
        self.list_b_done_in_7.setAlignment(QtCore.Qt.AlignCenter)
        self.list_b_done_in_7.setReadOnly(True)
        self.list_b_done_in_7.setObjectName("list_b_done_in_7")
        self.horizontalLayout_87.addWidget(self.list_b_done_in_7)
        self.list_b_label_7 = QtWidgets.QLabel(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.list_b_label_7.sizePolicy().hasHeightForWidth())
        self.list_b_label_7.setSizePolicy(sizePolicy)
        self.list_b_label_7.setMaximumSize(QtCore.QSize(110, 16777215))
        font = QtGui.QFont()
        # font.setUnderline(True)
        self.list_b_label_7.setFont(font)
        self.list_b_label_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.list_b_label_7.setObjectName("list_b_label_7")
        self.horizontalLayout_87.addWidget(self.list_b_label_7)
        self.verticalLayout_23.addLayout(self.horizontalLayout_87)
        self.horizontalLayout_94 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_94.setSpacing(12)
        self.horizontalLayout_94.setObjectName("horizontalLayout_94")
        self.malag_of_in = QtWidgets.QDoubleSpinBox(self.frame_3)
        self.malag_of_in.setRange(0, 1000)
        self.malag_of_in.setDecimals(1)
        self.malag_of_in.setSingleStep(0.5)
        self.malag_of_in.setAlignment(QtCore.Qt.AlignCenter)
        self.malag_of_in.setButtonSymbols(QtWidgets.QAbstractSpinBox.NoButtons)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.malag_of_in.sizePolicy().hasHeightForWidth())
        self.malag_of_in.setSizePolicy(sizePolicy)
        self.malag_of_in.setMaximumSize(QtCore.QSize(75, 16777215))
        self.malag_of_in.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.malag_of_in.setAlignment(QtCore.Qt.AlignCenter)
        self.malag_of_in.setObjectName("malag_of_in")
        self.horizontalLayout_94.addWidget(self.malag_of_in)
        self.malag_done_in = QtWidgets.QLineEdit(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.malag_done_in.sizePolicy().hasHeightForWidth())
        self.malag_done_in.setSizePolicy(sizePolicy)
        self.malag_done_in.setMaximumSize(QtCore.QSize(75, 16777215))
        self.malag_done_in.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.malag_done_in.setText("")
        self.malag_done_in.setAlignment(QtCore.Qt.AlignCenter)
        self.malag_done_in.setReadOnly(True)
        self.malag_done_in.setObjectName("malag_done_in")
        self.horizontalLayout_94.addWidget(self.malag_done_in)
        self.malag_label_7 = QtWidgets.QLabel(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.malag_label_7.sizePolicy().hasHeightForWidth())
        self.malag_label_7.setSizePolicy(sizePolicy)
        self.malag_label_7.setMaximumSize(QtCore.QSize(110, 16777215))
        self.malag_label_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.malag_label_7.setFont(font)
        self.malag_label_7.setObjectName("malag_label_7")
        self.horizontalLayout_94.addWidget(self.malag_label_7)
        self.verticalLayout_23.addLayout(self.horizontalLayout_94)
        self.horizontalLayout_90 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_90.setSpacing(12)
        self.horizontalLayout_90.setObjectName("horizontalLayout_90")
        self.free_of_in_7 = QtWidgets.QDoubleSpinBox(self.frame_3)
        self.free_of_in_7.setRange(0, 1000)
        self.free_of_in_7.setDecimals(1)
        self.free_of_in_7.setSingleStep(0.5)
        self.free_of_in_7.setAlignment(QtCore.Qt.AlignCenter)
        self.free_of_in_7.setButtonSymbols(QtWidgets.QAbstractSpinBox.NoButtons)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.free_of_in_7.sizePolicy().hasHeightForWidth())
        self.free_of_in_7.setSizePolicy(sizePolicy)
        self.free_of_in_7.setMaximumSize(QtCore.QSize(75, 16777215))
        self.free_of_in_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.free_of_in_7.setAlignment(QtCore.Qt.AlignCenter)
        self.free_of_in_7.setObjectName("free_of_in_7")
        self.horizontalLayout_90.addWidget(self.free_of_in_7)
        self.free_done_in_7 = QtWidgets.QLineEdit(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.free_done_in_7.sizePolicy().hasHeightForWidth())
        self.free_done_in_7.setSizePolicy(sizePolicy)
        self.free_done_in_7.setMaximumSize(QtCore.QSize(75, 16777215))
        self.free_done_in_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.free_done_in_7.setText("")
        self.free_done_in_7.setAlignment(QtCore.Qt.AlignCenter)
        self.free_done_in_7.setReadOnly(True)
        self.free_done_in_7.setObjectName("free_done_in_7")
        self.horizontalLayout_90.addWidget(self.free_done_in_7)
        self.free_label_7 = QtWidgets.QLabel(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.free_label_7.sizePolicy().hasHeightForWidth())
        self.free_label_7.setSizePolicy(sizePolicy)
        self.free_label_7.setMaximumSize(QtCore.QSize(110, 16777215))
        self.free_label_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.free_label_7.setFont(font)
        self.free_label_7.setObjectName("free_label_7")
        self.horizontalLayout_90.addWidget(self.free_label_7)
        self.verticalLayout_23.addLayout(self.horizontalLayout_90)
        self.horizontalLayout_88 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_88.setSpacing(12)
        self.horizontalLayout_88.setObjectName("horizontalLayout_88")
        self.project_of_in_7 = QtWidgets.QDoubleSpinBox(self.frame_3)
        self.project_of_in_7.setRange(0, 1000)
        self.project_of_in_7.setDecimals(1)
        self.project_of_in_7.setSingleStep(0.5)
        self.project_of_in_7.setAlignment(QtCore.Qt.AlignCenter)
        self.project_of_in_7.setButtonSymbols(QtWidgets.QAbstractSpinBox.NoButtons)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.project_of_in_7.sizePolicy().hasHeightForWidth())
        self.project_of_in_7.setSizePolicy(sizePolicy)
        self.project_of_in_7.setMaximumSize(QtCore.QSize(75, 16777215))
        self.project_of_in_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.project_of_in_7.setAlignment(QtCore.Qt.AlignCenter)
        self.project_of_in_7.setObjectName("project_of_in_7")
        self.horizontalLayout_88.addWidget(self.project_of_in_7)
        self.project_done_in_7 = QtWidgets.QLineEdit(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.project_done_in_7.sizePolicy().hasHeightForWidth())
        self.project_done_in_7.setSizePolicy(sizePolicy)
        self.project_done_in_7.setMaximumSize(QtCore.QSize(75, 16777215))
        self.project_done_in_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.project_done_in_7.setText("")
        self.project_done_in_7.setAlignment(QtCore.Qt.AlignCenter)
        self.project_done_in_7.setReadOnly(True)
        self.project_done_in_7.setObjectName("project_done_in_7")
        self.horizontalLayout_88.addWidget(self.project_done_in_7)
        self.project_label_7 = QtWidgets.QLabel(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.project_label_7.sizePolicy().hasHeightForWidth())
        self.project_label_7.setSizePolicy(sizePolicy)
        self.project_label_7.setMaximumSize(QtCore.QSize(110, 16777215))
        font = QtGui.QFont()
        # font.setUnderline(True)
        self.project_label_7.setFont(font)
        self.project_label_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.project_label_7.setObjectName("project_label_7")
        self.horizontalLayout_88.addWidget(self.project_label_7)
        self.verticalLayout_23.addLayout(self.horizontalLayout_88)
        self.horizontalLayout_89 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_89.setSpacing(12)
        self.horizontalLayout_89.setObjectName("horizontalLayout_89")
        self.sport_of_in_7 = QtWidgets.QDoubleSpinBox(self.frame_3)
        self.sport_of_in_7.setRange(0, 1000)
        self.sport_of_in_7.setDecimals(1)
        self.sport_of_in_7.setSingleStep(0.5)
        self.sport_of_in_7.setAlignment(QtCore.Qt.AlignCenter)
        self.sport_of_in_7.setButtonSymbols(QtWidgets.QAbstractSpinBox.NoButtons)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sport_of_in_7.sizePolicy().hasHeightForWidth())
        self.sport_of_in_7.setSizePolicy(sizePolicy)
        self.sport_of_in_7.setMaximumSize(QtCore.QSize(75, 16777215))
        self.sport_of_in_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.sport_of_in_7.setAlignment(QtCore.Qt.AlignCenter)
        self.sport_of_in_7.setToolTip("* Sport is a part of mandatory courses")
        self.sport_of_in_7.setObjectName("sport_of_in_7")
        self.horizontalLayout_89.addWidget(self.sport_of_in_7)
        self.sport_done_in_7 = QtWidgets.QLineEdit(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sport_done_in_7.sizePolicy().hasHeightForWidth())
        self.sport_done_in_7.setSizePolicy(sizePolicy)
        self.sport_done_in_7.setMaximumSize(QtCore.QSize(75, 16777215))
        self.sport_done_in_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.sport_done_in_7.setText("")
        self.sport_done_in_7.setAlignment(QtCore.Qt.AlignCenter)
        self.sport_done_in_7.setReadOnly(True)
        self.sport_done_in_7.setObjectName("sport_done_in_7")
        self.horizontalLayout_89.addWidget(self.sport_done_in_7)
        self.sport_label_7 = QtWidgets.QLabel(self.frame_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sport_label_7.sizePolicy().hasHeightForWidth())
        self.sport_label_7.setSizePolicy(sizePolicy)
        self.sport_label_7.setMaximumSize(QtCore.QSize(110, 16777215))
        self.sport_label_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.sport_label_7.setFont(font)
        self.sport_label_7.setObjectName("sport_label_7")
        self.horizontalLayout_89.addWidget(self.sport_label_7)
        self.verticalLayout_23.addLayout(self.horizontalLayout_89)
        self.horizontalLayout_91 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_91.setSpacing(12)
        self.horizontalLayout_91.setObjectName("horizontalLayout_91")
        spacerItem3 = QtWidgets.QSpacerItem(124, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_91.addItem(spacerItem3)
        self.english_checkbox_7 = QtWidgets.QCheckBox(self.frame_3)
        self.english_checkbox_7.setMaximumSize(QtCore.QSize(190, 16777215))
        self.english_checkbox_7.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.english_checkbox_7.setObjectName("english_checkbox_7")
        self.horizontalLayout_91.addWidget(self.english_checkbox_7)
        self.verticalLayout_23.addLayout(self.horizontalLayout_91)
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_23.addItem(spacerItem4)
        self.gridLayout_8.addLayout(self.verticalLayout_23, 0, 0, 1, 1)
        self.horizontalLayout_44.addWidget(self.frame_3)
        self.horizontalLayout_44.addWidget(self.frame)
        self.gridLayout_10.addLayout(self.horizontalLayout_44, 1, 0, 1, 1)
        self.horizontalLayout_93 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_93.setSpacing(0)
        self.horizontalLayout_93.setObjectName("horizontalLayout_93")
        self.horizontalLayout_45 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_45.setSpacing(7)
        self.horizontalLayout_45.setObjectName("horizontalLayout_45")
        spacerItem5 = QtWidgets.QSpacerItem(490, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_45.addItem(spacerItem5)
        self.deg_points_in = QtWidgets.QDoubleSpinBox(self.frame_11)
        self.deg_points_in.setRange(0, 1000)
        self.deg_points_in.setDecimals(1)
        self.deg_points_in.setSingleStep(0.5)
        self.deg_points_in.setAlignment(QtCore.Qt.AlignCenter)
        self.deg_points_in.setButtonSymbols(QtWidgets.QAbstractSpinBox.NoButtons)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.deg_points_in.sizePolicy().hasHeightForWidth())
        self.deg_points_in.setSizePolicy(sizePolicy)
        self.deg_points_in.setMaximumSize(QtCore.QSize(75, 16777215))
        self.deg_points_in.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.deg_points_in.setAlignment(QtCore.Qt.AlignCenter)
        self.deg_points_in.setReadOnly(False)
        self.deg_points_in.setObjectName("deg_points_in")
        self.horizontalLayout_45.addWidget(self.deg_points_in)
        self.deg_points_label = QtWidgets.QLabel(self.frame_11)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.deg_points_label.sizePolicy().hasHeightForWidth())
        self.deg_points_label.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setUnderline(True)
        self.deg_points_label.setFont(font)
        self.deg_points_label.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.deg_points_label.setObjectName("deg_points_label")
        self.horizontalLayout_45.addWidget(self.deg_points_label)
        self.horizontalLayout_93.addLayout(self.horizontalLayout_45)
        spacerItem6 = QtWidgets.QSpacerItem(21, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_93.addItem(spacerItem6)
        self.gridLayout_10.addLayout(self.horizontalLayout_93, 0, 0, 1, 1)
        self.verticalLayout_15.addWidget(self.frame_11)
        self.horizontalLayout_22.addLayout(self.verticalLayout_15)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.courses_tab_widget = QtWidgets.QTabWidget(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding,
                                           QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.courses_tab_widget.sizePolicy().hasHeightForWidth())
        self.courses_tab_widget.setSizePolicy(sizePolicy)
        self.courses_tab_widget.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.courses_tab_widget.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.courses_tab_widget.setTabsClosable(True)
        self.courses_tab_widget.setObjectName("courses_tab_widget")

        self.horizontalLayout.addWidget(self.courses_tab_widget)
        self.horizontalLayout_22.addLayout(self.horizontalLayout)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap("images/list-add.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.add_semester_but = QtWidgets.QPushButton(self.centralwidget)
        self.add_semester_but.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.add_semester_but.setIcon(icon1)
        self.add_semester_but.setObjectName("add_semester_but")
        self.add_semester_but.setToolTip("Add semester")
        self.courses_tab_widget.setCornerWidget(self.add_semester_but)
        # self.verticalLayout_4.addWidget(self.add_semester_but)
        self.gridLayout_3.addLayout(self.horizontalLayout_22, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1589, 36))
        self.menubar.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.menubar.setObjectName("menubar")
        self.menu = QtWidgets.QMenu(self.menubar)
        self.menu.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.menu.setObjectName("menu")
        self.menu_2 = QtWidgets.QMenu(self.menubar)
        self.menu_2.setObjectName("menu_2")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionNew = QtWidgets.QAction(MainWindow)
        self.actionNew.setObjectName("actionNew")
        self.actionLoadTemplate = QtWidgets.QAction(MainWindow)
        self.actionLoadTemplate.setObjectName("actionLoadTemplate")
        self.actionSave = QtWidgets.QAction(MainWindow)
        self.actionSave.setObjectName("actionSave")
        self.actionUpdate_Courses_DB = QtWidgets.QAction(MainWindow)
        self.actionUpdate_Courses_DB.setObjectName("actionUpdate_Courses_DB")
        self.action = QtWidgets.QAction(MainWindow)
        self.action.setObjectName("action")
        self.actionEnglish = QtWidgets.QAction(MainWindow)
        self.actionEnglish.setObjectName("actionEnglish")
        self.actionHeb = QtWidgets.QAction(MainWindow)
        self.actionHeb.setObjectName("actionInfo")
        self.actionNew = QtWidgets.QAction(MainWindow)
        self.actionNew.setObjectName("actionNew")
        self.actionSave = QtWidgets.QAction(MainWindow)
        self.actionSave.setObjectName("actionSave")
        self.actionSaveAs = QtWidgets.QAction(MainWindow)
        self.actionSaveAs.setObjectName("actionSaveAs")
        self.actionLoad = QtWidgets.QAction(MainWindow)
        self.actionLoad.setObjectName("actionLoad")
        self.actionUpdate_Courses_DB = QtWidgets.QAction(MainWindow)
        self.actionUpdate_Courses_DB.setObjectName("actionUpdate_Courses_DB")
        self.actionInfo = QtWidgets.QAction(MainWindow)
        self.actionInfo.setObjectName("actionInfo")
        self.actionNew.setShortcut("CTRL+N")
        self.actionLoadTemplate.setShortcut("CTRL+SHIFT+T")
        self.actionSave.setShortcut("CTRL+S")
        self.actionSaveAs.setShortcut("CTRL+SHIFT+S")
        self.actionLoad.setShortcut("CTRL+O")
        self.menu.addAction(self.actionNew)
        self.menu.addAction(self.actionSave)
        self.menu.addAction(self.actionSaveAs)
        self.menu.addAction(self.actionLoad)
        self.menu.addAction(self.actionLoadTemplate)
        self.menu.addSeparator()
        self.menu.addAction(self.actionUpdate_Courses_DB)
        self.menu.addAction(self.actionInfo)
        self.menu_2.addAction(self.actionEnglish)
        self.menu_2.addAction(self.actionHeb)
        self.menubar.addAction(self.menu.menuAction())
        self.menubar.addAction(self.menu_2.menuAction())

        self.retranslateUi(MainWindow)
        self.courses_tab_widget.setCurrentIndex(0)
        self.add_semester_but.clicked.connect(self.courses_tab_widget.update)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)
        MainWindow.setTabOrder(self.deg_points_in, self.add_semester_but)
        MainWindow.setTabOrder(self.sport_done_in_7, self.project_done_in_7)
        MainWindow.setTabOrder(self.project_done_in_7, self.must_done_in)
        MainWindow.setTabOrder(self.must_done_in, self.list_a_done_in_7)
        MainWindow.setTabOrder(self.list_a_done_in_7, self.average_in_7)
        MainWindow.setTabOrder(self.average_in_7, self.points_in_7)
        MainWindow.setTabOrder(self.points_in_7, self.points_left_in_7)
        MainWindow.setTabOrder(self.points_left_in_7, self.points_left_to_choose_in_7)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Degree Planer"))
        self.progress_label_8.setText(_translate("MainWindow", "Degree progress summary:"))
        self.desc_label.setText(_translate("MainWindow", "Summary:"))
        self.average_lab_7.setText(_translate("MainWindow", "Average:"))
        self.points_label_7.setText(_translate("MainWindow", "Points done:"))
        self.points_left_label_7.setText(_translate("MainWindow", "Points left:"))
        self.points_left_to_choose_label_7.setText(_translate("MainWindow", "Not chosen:"))
        self.done_label_7.setText(_translate("MainWindow", "Left:"))
        self.of_label_7.setToolTip(
            _translate("MainWindow", "<html><head/><body><p>You have to fill this column.</p></body></html>"))
        self.of_label_7.setText(_translate("MainWindow", "Of:"))
        self.must_of_in.setValue(0)
        self.must_label.setText(_translate("MainWindow", "Mandatory:"))
        self.list_a_of_in_7.setValue(0)
        self.list_a_label_7.setText(_translate("MainWindow", "A\' list:"))
        self.list_b_of_in_7.setValue(0)
        self.list_b_label_7.setText(_translate("MainWindow", "B\' list:"))
        self.malag_of_in.setValue(0)
        self.malag_label_7.setText(_translate("MainWindow", "Humanities:"))
        self.free_of_in_7.setValue(0)
        self.free_label_7.setText(_translate("MainWindow", "Free choice:"))
        self.project_of_in_7.setValue(0)
        self.project_label_7.setText(_translate("MainWindow", "Projects:"))
        self.sport_of_in_7.setValue(0)
        self.sport_label_7.setText(_translate("MainWindow", "Sport:"))
        self.english_checkbox_7.setText(_translate("MainWindow", "English\'s Exemption:"))
        self.deg_points_in.setValue(0)
        self.deg_points_label.setToolTip(_translate("MainWindow", "<html><head/><body><p>Fill!</p></body></html>"))
        self.deg_points_label.setText(_translate("MainWindow", "Degree\'s points:"))
        self.menu.setTitle(_translate("MainWindow", "Menu"))
        self.menu_2.setTitle(_translate("MainWindow", "Language"))
        self.actionEnglish.setText(_translate("MainWindow", "English"))
        self.actionHeb.setText(_translate("MainWindow", "עברית"))
        self.actionNew.setText(_translate("MainWindow", "New"))
        self.actionSave.setText(_translate("MainWindow", "Save"))
        self.actionSaveAs.setText(_translate("MainWindow", "Save As"))
        self.actionLoad.setText(_translate("MainWindow", "Load"))
        self.actionLoadTemplate.setText(_translate("MainWindow", "Load Template"))
        self.actionUpdate_Courses_DB.setText(_translate("MainWindow", "Update Courses"))
        self.actionInfo.setText(_translate("MainWindow", "Info"))


def loadDimensions():
    with open("settings.json", "r") as read_file:
        data = json.load(read_file)
        width = data['dimensions'][0]['width']
        height = data['dimensions'][0]['height']
        return width, height
    return 1320, 565



import json
import threading

from time import sleep

from PyQt5 import QtCore, QtWidgets

import Ui_tab_heb
import Ui_main_heb
from scrapper import *
from Ui_main_eng import Ui_MainWindow
from Ui_progress import ProgressWindowForm
from Ui_findDialog import Ui_course_search
from Ui_tab_eng import TabPage, createRemoveLineButton, createComboBox


class MyWindow(QtWidgets.QMainWindow):
    def __init__(self):
        super(MyWindow, self).__init__()
        self.progressBar = QtWidgets.QWidget()
        self.progress_ui = ProgressWindowForm()
        initDB()
        self.searchWindow = None
        self.update_allowed = True
        self.english_ui = self.loadLanguage()
        if self.english_ui:
            self.ui = Ui_MainWindow()
        else:
            self.ui = Ui_main_heb.Ui_MainWindow()
        self.ui.setupUi(self)
        self.not_show_remove_course = False
        self.Form = None
        self.not_show_remove_semester = False
        self.threadStop = [False]
        self.saveFileName = ''
        self.firstStart = True
        self.saved = True
        self.db_pairs = loadCourseNameNumberPairs()
        self.course_update = [0]
        self.ui.statusbar.setStyleSheet("font: 57 8pt \"Noto Sans\";")
        self.ui.statusbar.showMessage("© 2019 Vladimir Parakhin")
        self.update()
        self.show()
        self.setEventHandlers()
        self.thread_update = None
        self.thread_stop_checker = None
        self.search_ui = None
        try:
            file = open('settings.json', "r")
            if file:
                data = json.load(file)
                if '.dps' in data['save_file']:
                    if self.loadData(data['save_file']):
                        self.saveFileName = data['save_file']
                        file.close()
                    else:
                        data['save_file'] = ""
                        file.close()
                        file = open('settings.json', "w")
                        json.dump(data, file, indent=4)
                        file.close()

                if self.ui.courses_tab_widget.count() == 0:
                    self.addSemester()
                    self.saved = True
        except FileNotFoundError:
            if self.ui.courses_tab_widget.count() == 0:
                self.addSemester()
            self.update_allowed = True
            self.saved = True
            pass
        self.firstStart = False

    # Closing main windows override, check if there is unsaved data and prompts the user if so for save
    def closeEvent(self, event):
        if self.english_ui:
            quit_msg = "Exit"
        else:
            quit_msg = "יציאה"
        message_box = QtWidgets.QMessageBox()
        message_box.setWindowTitle(quit_msg)
        if self.english_ui:
            message_box.setText("Do you want to save your changes?")
        else:
            message_box.setText("לשמור לפני יציאה ?")
        message_box.setStandardButtons(
            QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No | QtWidgets.QMessageBox.Cancel)
        if not self.saved:
            reply = message_box.exec_()
            if reply == QtWidgets.QMessageBox.Yes:
                self.saveData()
            if reply == QtWidgets.QMessageBox.Cancel:
                event.ignore()
                return
        if self.searchWindow and self.searchWindow.isVisible():
            self.searchWindow.close()
        self.saveWindowDimensions()
        return super().closeEvent(event)

    def saveWindowDimensions(self):
        filename = open("settings.json", "r")
        if filename:
            data = json.load(filename)
            filename.close()
            data['dimensions'][0]['width'] = self.width()
            data['dimensions'][0]['height'] = self.height()
            with open("settings.json", "w") as write_file:
                json.dump(data, write_file, indent=4)

    @staticmethod
    def loadLanguage():
        with open("settings.json", "r") as read_file:
            data = json.load(read_file)
            if data['language'] == 'heb':
                return False
            elif data['language'] == 'eng':
                return True
        return False

    def languageChange(self, language):
        if self.english_ui and language == 'eng':
            return
        elif not self.english_ui and language == 'heb':
            return
        elif self.english_ui and language == 'heb':
            data = self.extractData()
            self.english_ui = False
            self.ui = Ui_main_heb.Ui_MainWindow()
            self.ui.setupUi(self)
            with open("settings.json", "r+") as read_file:
                data_json = json.load(read_file)
                data_json['language'] = 'heb'
                read_file.seek(0)
                json.dump(data_json, read_file, indent=4)
                read_file.truncate()
        elif not self.english_ui and language == 'eng':
            data = self.extractData()
            self.english_ui = True
            self.ui = Ui_MainWindow()
            self.ui.setupUi(self)
            with open("settings.json", "r+") as read_file:
                data_json = json.load(read_file)
                data_json['language'] = 'eng'
                read_file.seek(0)
                json.dump(data_json, read_file, indent=4)
                read_file.truncate()
        if self.searchWindow and self.searchWindow.isVisible():
            self.searchWindow.close()
            self.openSearchDialog()
        self.setEventHandlers()
        self.loadData('', data)

    def setEventHandlers(self):
        self.ui.english_checkbox_7.stateChanged['int'].connect(self.update)
        self.ui.actionUpdate_Courses_DB.triggered.connect(self.dbUpdate)
        self.ui.actionNew.triggered.connect(self.clearData)
        self.ui.actionSave.triggered.connect(self.saveData)
        self.ui.actionSaveAs.triggered.connect(self.saveAsData)
        self.ui.actionLoad.triggered.connect(self.loadData)
        self.ui.actionLoadTemplate.triggered.connect(self.loadTemplate)
        self.ui.actionInfo.triggered.connect(self.showCredit)
        self.ui.add_semester_but.clicked.connect(self.addSemester)
        self.ui.actionHeb.triggered.connect(lambda: self.languageChange('heb'))
        self.ui.actionEnglish.triggered.connect(lambda: self.languageChange('eng'))
        self.ui.courses_tab_widget.tabCloseRequested.connect(
            self.removeSemester)
        self.ui.must_of_in.valueChanged.connect(self.update)
        self.ui.list_a_of_in_7.valueChanged.connect(self.update)
        self.ui.list_b_of_in_7.valueChanged.connect(self.update)
        self.ui.project_of_in_7.valueChanged.connect(self.update)
        self.ui.sport_of_in_7.valueChanged.connect(self.update)
        self.ui.malag_of_in.valueChanged.connect(self.update)
        self.ui.free_of_in_7.valueChanged.connect(self.update)
        self.ui.deg_points_in.valueChanged.connect(self.update)

    # Function which launches data base update, pops progress bar window
    def dbUpdate(self):
        update_time = "Never"
        with open('settings.json', 'r') as write_file:
            data_json = json.load(write_file)
            update_time = data_json['db-updated']
        if self.english_ui:
            if not self.warningMsg(title="Warning", msg="Data-base update may take approximately 10 minutes.\n\n" + "Last time updated at: " + update_time + "\n\nProceed?"):
                return
        else:
            if update_time == "Never":
                update_time = "אין נתונים"
            if not self.warningMsg(title="התראה", msg="עדכון קורסים עלול לקחת 10 דק'.\n\n" + "הקורסים עודכנו לאחרונה בתאריך: " + update_time + "\n\nלהמשיך?"):
                return
        self.progress_ui.setupUi(self.progressBar, self.english_ui)
        self.progressBar.children()[1].children()[3].clicked.connect(self.setStopThread)
        self.progressBar.show()
        self.course_update[0] = 0
        self.threadStop[0] = False
        for tab in range(self.ui.courses_tab_widget.count()):
            self.ui.courses_tab_widget.widget(tab).children()[8].setEnabled(False)  # search button
            if self.english_ui:
                self.ui.courses_tab_widget.widget(tab).children()[8].setToolTip(
                    "Data base update in progress, please wait.")
            else:
                self.ui.courses_tab_widget.widget(tab).children()[8].setToolTip("אמתן לסיום עדכון הנתונים ")
        self.progress_ui.progressBar.setValue(0)
        self.thread_update = threading.Thread(target=updateDb,
                                              args=[self, self.course_update, self.progress_ui, self.threadStop])
        self.thread_stop_checker = threading.Thread(target=self.stopSearch, args=[])
        self.thread_update.start()
        self.thread_stop_checker.start()

    # Function to set a signal for update db thread to stop
    def setStopThread(self):
        self.threadStop[0] = True

    # Function which check whether update db thread finished and restores find course button functionality
    def stopSearch(self):
        while not self.threadStop[0] and self.thread_update.isAlive():
            sleep(0.05)
        self.progressBar.close()
        self.progressBar.destroy()
        for tab in range(self.ui.courses_tab_widget.count()):
            self.ui.courses_tab_widget.widget(tab).children()[8].setEnabled(True)
            self.ui.courses_tab_widget.widget(tab).children()[8].setToolTip("")

    # Clearing the UI input completely 
    def clearData(self, forced=False):
        if not forced:
            if self.english_ui:
                answer = self.warningMsg(not_show_param='', msg="Remove all the data?")
            else:
                answer = self.warningMsg(not_show_param='', msg="למחוק את כל הנתונים?")
        if forced or answer:
            for tab in range(self.ui.courses_tab_widget.count() - 1, -1, -1):
                self.removeSemester(tab, force=True)
            self.ui.project_of_in_7.setValue(0.0)
            self.ui.list_a_of_in_7.setValue(0.0)
            self.ui.list_b_of_in_7.setValue(0.0)
            self.ui.sport_of_in_7.setValue(0.0)
            self.ui.malag_of_in.setValue(0.0)
            self.ui.deg_points_in.setValue(0.0)
            self.ui.free_of_in_7.setValue(0.0)
            self.ui.must_of_in.setValue(0.0)
            self.ui.english_checkbox_7.setChecked(False)
            self.update()
            self.saveFileName=''
        if self.ui.courses_tab_widget.count() == 0:
            self.addSemester()

    # Load saved data from file using pickle module
    def loadData(self, filename='', data=None):
        if data is None:
            data = []
        if len(data) == 0:
            self.update_allowed = False
            if not self.firstStart and not self.saved:
                if self.english_ui:
                    answer = self.warningMsg(not_show_param='', msg="Do you want save all the changes?")
                else:
                    answer = self.warningMsg(not_show_param='', msg="שינויים שלא נשמרו ימחקו, לשמור?")
                if answer:
                    self.saveData()
            if filename == '' or not filename:
                filename = QtWidgets.QFileDialog.getOpenFileName(self, 'טעינה', 'saves', "Save files (*.dps)")
                if filename[0] == '':
                    self.update_allowed = True
                    return False
                filename = filename[0]
            try:
                file_path = filename
                filename = open(filename, "rb")
            except FileNotFoundError:
                if self.english_ui:
                    self.errorMsg("Couldn't open the file")
                else:
                    self.errorMsg("פתיחת קובץ כשלה")
                self.update_allowed = True
                return False
        self.clearData(True)
        # The data is stored in the following way 
        # content[0] first list contains values from the points value which the user stored
        index = 0
        try:
            if len(data) == 0:
                content = pickle.loads(filename.read())
            else:
                content = data
            self.ui.deg_points_in.setValue(content[0][0])
            self.ui.must_of_in.setValue(content[0][1])
            self.ui.list_a_of_in_7.setValue(content[0][2])
            self.ui.list_b_of_in_7.setValue(content[0][3])
            self.ui.project_of_in_7.setValue(content[0][4])
            self.ui.malag_of_in.setValue(content[0][5])
            self.ui.sport_of_in_7.setValue(content[0][6])
            self.ui.free_of_in_7.setValue(content[0][7])
            self.ui.english_checkbox_7.setChecked(content[0][8])
            content = content[1:]
            # The other data saved in the following order
            # [i]-semester
            # [i][j]-row
            # [i][j][k]-column
            first_sem = True
            for semester in content:
                if not first_sem:
                    self.addSemester()
                first_sem = False
                if self.english_ui:
                    table = self.ui.courses_tab_widget.widget(index).children()[1]
                else:
                    table = self.ui.courses_tab_widget.widget(index).children()[7]
                row_index = 0
                while len(semester) > table.rowCount():
                    self.addRow(table)
                while len(semester) < table.rowCount():
                    self.removeRow(table)
                for row in semester:
                    for column in range(0, len(row) - 1):
                        if column == 0:
                            item = QtWidgets.QTableWidgetItem()
                            item.setTextAlignment(QtCore.Qt.AlignCenter)
                            table.setItem(row_index, column, item)
                            if self.english_ui:
                                table.setCellWidget(row_index, column, createComboBox())
                            else:
                                table.setCellWidget(row_index, column, Ui_tab_heb.createComboBox())
                            table.cellWidget(row_index, column).currentIndexChanged.connect(self.update)
                            table.cellWidget(row_index, column).setCurrentIndex(int(row[column]))
                        # Course number
                        elif column == 1:
                            if row[column] != ' ' and row[column] != ' \n':
                                # Course dependencies (parallel, similar, inclusive)
                                table.item(row_index, column).setToolTip(row[column])
                            if row[column + 1] != ' ' and row[column + 1] != ' \n':
                                table.item(row_index, column).setText(row[column + 1])
                        # Course name
                        elif column == 2:
                            if row[column + 1] != ' ' and row[column + 1] != ' \n':
                                table.item(row_index, column).setText(row[column + 1])
                        else:
                            if row[column + 1] != ' ' and row[column + 1] != ' \n':
                                try:
                                    # Course-points spin-box
                                    if column == 3:
                                        spin_box = QtWidgets.QDoubleSpinBox()
                                        spin_box.setDecimals(1)
                                        spin_box.setSingleStep(0.5)
                                    # Course-grade spin-box
                                    else:
                                        spin_box = QtWidgets.QSpinBox()
                                        spin_box.setSingleStep(1)
                                    spin_box.setRange(0, 100)
                                    spin_box.setAlignment(QtCore.Qt.AlignCenter)
                                    spin_box.valueChanged.connect(self.update)
                                    spin_box.setButtonSymbols(QtWidgets.QAbstractSpinBox.NoButtons)
                                    table.setCellWidget(row_index, column, spin_box)
                                    table.cellWidget(row_index, column).valueChanged.connect(self.update)
                                    table.cellWidget(row_index, column).setValue(float(row[column + 1]))
                                except ValueError:
                                    table.cellWidget(row_index, column).setValue(0.0)
                    row_index += 1
                index += 1
            self.update_allowed = True
            if data is None:
                self.saveFileName = file_path
        except EOFError:
            if self.english_ui:
                self.errorMsg("File corrupted")
            else:
                self.errorMsg("קריאת הקובץ נכשלה")
        if self.ui.courses_tab_widget.count() == 0:
            self.addSemester()
        self.update()
        if len(data) == 0:
            self.saved = True
        return True

    # Load template of courses
    def loadTemplate(self):
        self.update_allowed = False
        if not self.firstStart and not self.saved:
            if self.english_ui:
                answer = self.warningMsg(not_show_param='', msg="Do you want save all the changes?")
            else:
                answer = self.warningMsg(not_show_param='', msg="שינויים שלא נשמרו ימחקו, לשמור?")
            if answer:
                self.saveData()
        if self.english_ui:
            filename = QtWidgets.QFileDialog.getOpenFileName(self, 'Load', 'templates', "Json-Templates files (*.json)")
        else:
            filename = QtWidgets.QFileDialog.getOpenFileName(self, 'טעינה', 'templates',
                                                             "Json-Templates files (*.json)")
        if filename[0] == '':
            self.update_allowed = True
            if self.english_ui:
                return
            else:
                return
        filename = filename[0]
        courses = []
        try:
            with open(filename, "r+") as read_file:
                data_json = json.load(read_file)
                total_points = data_json["Points"]
                mandatory = data_json["Mandatory"]
                a_list = data_json["A' List"]
                b_list = data_json["B' List"]
                humanistic = data_json["Humanistic"]
                free_points = data_json["Free choice"]
                sport = data_json["Sport"]
                for semester in data_json["Courses"][0]:
                    courses.append(data_json["Courses"][0][semester])
        except (KeyError, FileNotFoundError):
            self.update_allowed = True
            if self.english_ui:
                self.errorMsg("File corrupted")
            else:
                self.errorMsg("קריאת הקובץ נכשלה")
            return
        self.clearData(forced=True)
        if self.english_ui:
            table = self.ui.courses_tab_widget.currentWidget().children()[1]
        else:
            table = self.ui.courses_tab_widget.currentWidget().children()[7]
        first_sem = True
        for semester in courses:
            if not first_sem:
                self.addSemester()
            for course in semester:
                if course == "MATH":
                    math_course = Course()
                    math_course.name = "מתמטי נוסף"
                    math_course.number = "104"
                    self.addCourseContent(0, math_course)
                elif course == "SCI":
                    sci_course = Course()
                    sci_course.name = "קורס מדעי"
                    self.addCourseContent(0, sci_course)
                elif course == "SPORT":
                    sport_course = Course()
                    sport_course.name = "ספורט"
                    sport_course.points = 1
                    self.addCourseContent(0, sport_course)
                else:
                    self.addCourseContent(course_num=course)
            first_sem = False
        self.ui.deg_points_in.setValue(total_points)
        self.ui.must_of_in.setValue(mandatory)
        self.ui.list_a_of_in_7.setValue(a_list)
        self.ui.list_b_of_in_7.setValue(b_list)
        self.ui.project_of_in_7.setValue(0)
        self.ui.malag_of_in.setValue(humanistic)
        self.ui.sport_of_in_7.setValue(sport)
        self.ui.free_of_in_7.setValue(free_points)
        self.ui.english_checkbox_7.setChecked(False)
        self.update_allowed = True
        self.update()
        self.saved = True
        self.saveFileName = ''

    # Save as function change the pointer to current saved file
    def saveAsData(self):
        self.saveFileName = ''
        self.saveData()

    # Save function, saves the data from the app using pickle
    def saveData(self):
        if self.saveFileName == '':
            if self.english_ui:
                filename = QtWidgets.QFileDialog.getSaveFileName(self, 'Save as', 'saves', "Save files (*.dps)")
            else:
                filename = QtWidgets.QFileDialog.getSaveFileName(self, 'שמירה בשם', 'saves', "Save files (*.dps)")
            if filename[0] == '':
                return
            self.saveFileName = filename[0]
        try:
            file = open(self.saveFileName, "wb+")
        except FileNotFoundError:
            if self.english_ui:
                self.errorMsg("Couldn't open file.")
            else:
                self.errorMsg("פתיחת קובץ כשלה")
            return
        if not file:
            if self.english_ui:
                self.errorMsg("Couldn't open file.")
            else:
                self.errorMsg("פתיחת קובץ כשלה")
            return
        byte_array = self.extractData()
        file.write(pickle.dumps(byte_array))
        # Caching the filename in settings.json in order to use save function seamlessly afterwards
        with open('settings.json', 'r+') as write_file:
            data_json = json.load(write_file)
            data_json['save_file'] = self.saveFileName
            write_file.seek(0)
            json.dump(data_json, write_file, indent=4)
            write_file.truncate()
        self.saved = True

    # The function extracts all the user-input data and returns it in 3-dimensional array.
    def extractData(self):
        byte_array = list()
        inputs = list()
        inputs.append(self.ui.deg_points_in.value())
        inputs.append(self.ui.must_of_in.value())
        inputs.append(self.ui.list_a_of_in_7.value())
        inputs.append(self.ui.list_b_of_in_7.value())
        inputs.append(self.ui.project_of_in_7.value())
        inputs.append(self.ui.malag_of_in.value())
        inputs.append(self.ui.sport_of_in_7.value())
        inputs.append(self.ui.free_of_in_7.value())
        inputs.append(self.ui.english_checkbox_7.isChecked())
        byte_array.append(inputs)
        for tab in range(self.ui.courses_tab_widget.count()):
            if self.english_ui:
                table = self.ui.courses_tab_widget.widget(tab).children()[1]
            else:
                table = self.ui.courses_tab_widget.widget(tab).children()[7]
            semester = list()
            for row in range(table.rowCount()):
                rows = list()
                for column in range(table.columnCount() - 1):
                    if column == 0:
                        if table.cellWidget(row, column):
                            rows.append(table.cellWidget(row, column).currentIndex())
                    elif column <= 2:
                        if table.item(row, column):
                            if column == 1:
                                rows.append(table.item(row, column).toolTip())
                            rows.append(table.item(row, column).text())
                    else:
                        if table.cellWidget(row, column):
                            rows.append(table.cellWidget(row, column).value())
                semester.append(rows)
            byte_array.append(semester)
        return byte_array

    # Function which shows credit window with contact information
    def showCredit(self):
        message_det = """<address>
                        Degree Planner
                        Version: 1.0<br>
                        Contact details:<br> 
                        <a href='vov4ikpa@gmail.com'>\tEmail</a><br>
                        <a href='https://github.com/Vladimir-pa/Degree-Planner'>\tGithub</a><br>
                        © 2019 Vladimir Parakhin
                        </address>"""
        msgbox = QtWidgets.QMessageBox()
        msgbox.setText(message_det)
        msgbox.setWindowTitle("About")
        msgbox.setTextFormat(QtCore.Qt.RichText)
        size_policy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Maximum)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        msgbox.setSizePolicy(size_policy)
        msgbox.exec()

    # Function which opens the course search dialog in order to search within 
    #   the data base for courses and add them into the table
    def openSearchDialog(self):
        self.searchWindow = QtWidgets.QWidget()
        self.search_ui = Ui_course_search()
        self.search_ui.setupUi(self.searchWindow, self.english_ui)
        # db_pairs contains all courses from the technion for quick lookup
        # in the following format "<course-name> - <course-number>"
        for course in self.db_pairs:
            self.searchWindow.children()[3].addItem(course)
        self.searchWindow.children()[3].currentIndexChanged.connect(self.findCourse)
        self.searchWindow.children()[4].setChecked(True)
        self.searchWindow.children()[7].clicked.connect(lambda state: self.closeIt(self.searchWindow))  # close
        self.searchWindow.children()[8].clicked.connect(lambda state: self.addCourse(self.searchWindow))  # add
        if self.english_ui:
            self.searchWindow.setWindowTitle("Course search")
        else:
            self.searchWindow.setWindowTitle("חיפוש קורסים")
        self.searchWindow.children()[3].setCurrentText("")
        self.searchWindow.show()

    # Function which find the course in the data-base list in course search ui
    # the function is being called on index change of the spinbox with the course in the ui
    # the function updates the additional information about the course
    def findCourse(self):
        if self.searchWindow.children()[3].currentText() != '':
            course = findCourseInDB(self.searchWindow.children()[3].currentText().split(" - ")[0])
            if self.english_ui:
                course.english = True
            self.searchWindow.children()[6].setPlainText(repr(course))
        else:
            self.searchWindow.children()[6].setPlainText("")

    # Given a table and a row checks whether the row is empty
    def checkIfRowIsEmpty(self, table, row):
        for column in range(1, table.columnCount() - 1):
            if table.item(row, column):
                if column >= 3:
                    if table.cellWidget(row, column).value() > 0:
                        return False
                if table.item(row, column).text() != "":
                    return False
        return True

    # Function which adds the course content into the table on current open semester in an empty row
    def addCourseContent(self, course_num, course=None, table=None):
        if course is None:
            course = findCourseInDB(course_num)
        if type(course) != Course:
            print(course_num + "\n")
            return
        for i in range(0, self.ui.courses_tab_widget.count()):
            if self.english_ui:
                semester_table = self.ui.courses_tab_widget.widget(i).children()[1]
            else:
                semester_table = self.ui.courses_tab_widget.widget(i).children()[7]
            if self.courseInTable(semester_table, str(course.number)):
                if self.english_ui:
                    if not self.warningMsg(title="Warning", msg="Course " + str(
                            course.number) + " exist in the table, add again?" + "\n(In Semester" + str(i + 1) + ")"):
                        return
                else:
                    if not self.warningMsg(title="התראה",
                                           msg="הקורס " + str(
                                               course.number) + " קיים בטבלה, להוסיף שוב?" + "\n(בסמסטר " + str(
                                               i + 1) + ")"):
                        return
        if table is None:
            table = semester_table
        row = self.findEmptyRow(table)
        course_num = QtWidgets.QTableWidgetItem()
        if course and 'ספורט' in course.name:
            sport_number = 5
            table.cellWidget(row, 0).setCurrentIndex(sport_number)
        course_num.setText(str(course.number))
        course_name = QtWidgets.QTableWidgetItem()
        course_name.setText(course.name)
        course_points = QtWidgets.QTableWidgetItem()
        table.cellWidget(row, 3).setValue(course.points)
        course_name.setTextAlignment(QtCore.Qt.AlignCenter)
        course_num.setTextAlignment(QtCore.Qt.AlignCenter)
        if self.english_ui:
            tooltip = ("Pre-requisites: " + course.reprDependencies() + "\n" if course.reprDependencies() != "" else "") \
                      + ("Side courses: " + course.repOtherData(course.parallel) + "\n" if course.repOtherData(
                course.parallel) != "" else "") \
                      + ("Similar courses: " + course.repOtherData(
                course.similarities) + "\n" if course.repOtherData(course.similarities) != "" else "") \
                      + ("Similar courses(included): " + course.repOtherData(course.inclusive) if course.repOtherData(
                course.inclusive) != "" else "")
        else:
            tooltip = ("מקוצועות קדם: " + course.reprDependencies() + "\n" if course.reprDependencies() != "" else "") \
                      + ("מקצועות צמודים: " + course.repOtherData(course.parallel) + "\n" if course.repOtherData(
                course.parallel) != "" else "") \
                      + ("מקצועות ללא זיכוי נוסף: " + course.repOtherData(
                course.similarities) + "\n" if course.repOtherData(course.similarities) != "" else "") \
                      + ("מקצועות ללא זיכוי נוסף(מוכלים): " + course.repOtherData(
                course.inclusive) if course.repOtherData(
                course.inclusive) != "" else "")
        course_num.setToolTip(tooltip)
        course_points.setTextAlignment(QtCore.Qt.AlignCenter)
        table.setItem(row, 1, course_num)
        table.setItem(row, 2, course_name)
        return

    # Function which check whether or not the course with the given course number is present in the table in current
    # semester
    def courseInTable(self, table, course_num, semesters_number=-1):
        if semesters_number == -1:
            semesters_number = table.rowCount()
        for row in range(0, semesters_number):
            if table.item(row, 1) and table.item(row, 1).text() == course_num and course_num != '':
                return True
        return False

    #   Function which find an empty row in the current semester table
    def findEmptyRow(self, table):
        for row in range(table.rowCount()):
            if self.checkIfRowIsEmpty(table, row):
                return row
        self.addRow(table)
        return table.rowCount() - 1

    # Function which adds the course from the search ui on add-course button click event
    def addCourse(self, course_info):
        combo_text = course_info.children()[3].currentText()
        if combo_text == '':
            if self.english_ui:
                self.errorMsg("Please choose course and try again.")
            else:
                self.errorMsg("לא נבחר קורס, אנא נסה שנית")
            return
        course_number = combo_text.split(" - ")[0]
        if self.english_ui:
            table = self.ui.courses_tab_widget.currentWidget().children()[1]
        else:
            table = self.ui.courses_tab_widget.currentWidget().children()[7]
        if self.searchWindow.children()[4].isChecked():
            dependencies = self.checkDependencies(table, course_number, self.ui.courses_tab_widget.currentIndex())
            if len(dependencies) > 0:
                dependencies_msg = self.createDependenciesMessage(dependencies)
                self.prerequisitesWarningMessage(dependencies_msg)
        self.addCourseContent(course_number, table=table)

    def checkDependencies(self, table, course_number, current_semester):
        course = findCourseInDB(course_number)
        if "השלמות" in course.name:
            return True
        if len(course.dependencies) == 0 and len(course.parallel) == 0:
            return True
        dependencies = []
        parallels = []
        for option in course.dependencies:
            inner_dependencies = []
            for course_number in option:
                if not self.courseInTable(table, course_number, current_semester - 1):
                    inner_dependencies.append(course_number)
            if len(dependencies) == 0 or len(inner_dependencies) < len(dependencies):
                dependencies = inner_dependencies
        for course_number in course.parallel:
            if not self.courseInTable(table, course_number, current_semester - 1):
                parallels.append(course_number)
        return [dependencies, parallels]

    def createDependenciesMessage(self, dependencies):
        msg = ""
        if len(dependencies[0]) > 0:
            if self.english_ui:
                msg += "Following pre-requisites are missing:\n"
            else:
                msg += "חסרים הקדמים הבאים בסמסטרים קודמים:\n"
            separator = ""
            for course_number in dependencies[0]:
                course = findCourseInDB(course_number)
                msg += separator + course.number + " - " + course.name
                separator = "\n"
            msg += "\n\n"
        if len(dependencies[1]) > 0:
            if self.english_ui:
                msg += "Following parallel courses are missing:\n"
            else:
                msg += "חסרים הצמודים הבאים בסמסטר נוכחי או בסמסטרים קודמים:\n"
            separator = ""
            for course_number in dependencies[1]:
                course = findCourseInDB(course_number)
                msg += separator + course.number + " - " + course.name
                separator = "\n"
        msg += "\n\n"
        return msg

    # Function which closes the widget
    def closeIt(self, widget):
        widget.close()

    # Function which creates a new tab for a new semester
    def createTab(self):
        self.Form = QtWidgets.QWidget()
        if self.english_ui:
            ui = TabPage()
        else:
            ui = Ui_tab_heb.TabPage()
        ui.setupUi(self.Form)
        return self.Form

    # Function which is a part of update() function
    # the function calculates and updates the averages of each semester and the total average
    def updateAverage(self):
        total_sum = 0
        total_points = 0
        for tab in range(self.ui.courses_tab_widget.count()):
            semester_sum = 0
            semester_points = 0
            if self.english_ui:
                table = self.ui.courses_tab_widget.widget(tab).children()[1]
            else:
                table = self.ui.courses_tab_widget.widget(tab).children()[7]
            if self.english_ui:
                semester_average = self.ui.courses_tab_widget.widget(tab).children()[4]
            else:
                semester_average = self.ui.courses_tab_widget.widget(tab).children()[2]
            for row in range(table.rowCount()):
                try:
                    if table.cellWidget(row, 3) and float(table.cellWidget(row, 3).value()) > 0:
                        if table.cellWidget(row, 4) and float(table.cellWidget(row, 4).value()) > 0:
                            semester_points += float(table.cellWidget(row, 3).value())
                            semester_sum += float(table.cellWidget(row, 3).value()) * float(
                                table.cellWidget(row, 4).value())
                except (ValueError, AttributeError):
                    continue
            if semester_points > 0:
                semester_average.setText(str(round(semester_sum / semester_points, 2)))
            else:
                semester_average.setText(str(0.0))
            total_points += semester_points
            total_sum += semester_sum
        if total_points > 0:
            self.ui.average_in_7.setText(str(round(total_sum / total_points, 2)))
        else:
            self.ui.average_in_7.setText(str(0.0))

    # Function which is a part of update() function
    # the function calculates and updates the points of each semester and the total points information
    def updatePoints(self):
        if self.english_ui:
            points = {"Mandatory": 0,
                      "A' list": 0,
                      "B' list": 0,
                      "Project": 0,
                      "Sport": 0,
                      "Humanistic": 0,
                      "Free choice": 0,
                      }
        else:
            points = {"חובה": 0,
                      "רשימה א": 0,
                      "רשימה ב": 0,
                      "פרוייקט": 0,
                      "ספורט": 0,
                      "מל\"ג": 0,
                      "חופשי": 0}
        points_done = 0
        for tab in range(self.ui.courses_tab_widget.count()):
            if self.english_ui:
                table = self.ui.courses_tab_widget.widget(tab).children()[1]
                table_points = self.ui.courses_tab_widget.widget(tab).children()[6]  # Points
            else:
                table = self.ui.courses_tab_widget.widget(tab).children()[7]
                table_points = self.ui.courses_tab_widget.widget(tab).children()[4]  # Points
            table_points.setText("0")
            for row in range(table.rowCount()):
                try:
                    try:
                        table.cellWidget(row, 3).setText(str(float(table.cellWidget(row, 3).value())))
                    except (ValueError, AttributeError):
                        pass
                    if table.cellWidget(row, 3) and float(table.cellWidget(row, 3).value()) > 0:
                        try:
                            if table.cellWidget(row, 4) and float(table.cellWidget(row, 4).value()) > 0:
                                points_done += float(table.cellWidget(row, 3).value())
                        except ValueError:
                            pass
                        points[table.cellWidget(row, 0).currentText()] += float(table.cellWidget(row, 3).value())
                        table_points.setText(
                            str(round(float(table_points.text()) + float(table.cellWidget(row, 3).value()), 1)))
                except (ValueError, AttributeError, KeyError):
                    continue
        if self.english_ui:
            self.ui.list_a_done_in_7.setText(str(self.ui.list_a_of_in_7.value() - points["A' list"]))
            self.ui.list_b_done_in_7.setText(str(self.ui.list_b_of_in_7.value() - points["B' list"]))
            self.ui.project_done_in_7.setText(str(self.ui.project_of_in_7.value() - points["Project"]))
            self.ui.sport_done_in_7.setText(str(self.ui.sport_of_in_7.value() - points["Sport"]))
            self.ui.malag_done_in.setText(str(self.ui.malag_of_in.value() - points["Humanistic"]))
            self.ui.free_done_in_7.setText(str(self.ui.free_of_in_7.value() - points["Free choice"]))
        else:
            self.ui.list_a_done_in_7.setText(str(self.ui.list_a_of_in_7.value() - points["רשימה א"]))
            self.ui.list_b_done_in_7.setText(str(self.ui.list_b_of_in_7.value() - points["רשימה ב"]))
            self.ui.project_done_in_7.setText(str(self.ui.project_of_in_7.value() - points["פרוייקט"]))
            self.ui.sport_done_in_7.setText(str(self.ui.sport_of_in_7.value() - points["ספורט"]))
            self.ui.malag_done_in.setText(str(self.ui.malag_of_in.value() - points["מל\"ג"]))
            self.ui.free_done_in_7.setText(str(self.ui.free_of_in_7.value() - points["חופשי"]))
        if self.ui.english_checkbox_7.isChecked():
            exemption = 3
        else:
            exemption = 0
        if self.english_ui:
            self.ui.must_done_in.setText(str(float(self.ui.must_of_in.value() - points["Mandatory"] - exemption)))
        else:
            self.ui.must_done_in.setText(str(float(self.ui.must_of_in.value() - points["חובה"] - exemption)))
        self.ui.points_left_to_choose_in_7.setText(
            str(self.ui.deg_points_in.value() - sum(points.values()) - exemption))
        self.ui.points_in_7.setText(str(float(points_done) + exemption))
        self.ui.points_left_in_7.setText(str(self.ui.deg_points_in.value() - float(self.ui.points_in_7.text())))

    # Function which is a part of update() function
    # the function calculates and updates the tooltips for every course in the semester
    def updateTooltips(self):
        for tab in range(self.ui.courses_tab_widget.count()):
            if self.english_ui:
                table = self.ui.courses_tab_widget.widget(tab).children()[1]
            else:
                table = self.ui.courses_tab_widget.widget(tab).children()[7]
            for row in range(table.rowCount()):
                try:
                    if table.item(row, 1) and (table.item(row, 1).text() == '' or table.item(row, 1).text() == ' '):
                        table.item(row, 1).setToolTip("")
                    if table.item(row, 2) and not (table.item(row, 2).text() == '' or table.item(row, 2).text() == ' '):
                        table.item(row, 2).setToolTip(table.item(row, 2).text())
                except (ValueError, AttributeError):
                    continue

    # Update function which keeps all the data of the program updated
    def update(self):
        if self.update_allowed:
            try:
                self.updateAverage()
                self.updatePoints()
                self.updateTooltips()
                self.saved = False
            except (ValueError, AttributeError):
                pass

    # Add new empty semester to the program
    def addSemester(self):
        tab = self.createTab()
        if self.english_ui:
            table = tab.children()[1]
            tab.children()[2].clicked.connect(lambda state: self.addRow(
                table))
            tab.children()[3].clicked.connect(lambda state: self.removeRow(
                table))
        else:
            table = tab.children()[7]
            tab.children()[5].clicked.connect(lambda state: self.addRow(
                table))
            tab.children()[6].clicked.connect(lambda state: self.removeRow(
                table))
        table.cellChanged['int', 'int'].connect(self.update)
        tab.children()[8].clicked.connect(self.openSearchDialog)
        value = []
        lambdas = []
        buttons = []
        for row in range(0, table.rowCount()):
            table.cellWidget(row, 3).valueChanged.connect(self.update)  # Check whether course points were update
            table.cellWidget(row, 4).valueChanged.connect(self.update)  # Check whether course grade was updated
            if self.english_ui:
                button = createRemoveLineButton(str(row))
            else:
                button = Ui_tab_heb.createRemoveLineButton(str(row))
            value.append(row)
            lambdas.append(lambda state: self.clearRow(table))
            button.clicked.connect(lambdas[row])
            buttons.append(button)
            table.setCellWidget(row, table.columnCount() - 1, buttons[row])
            table.cellWidget(row, 0).currentIndexChanged.connect(self.update)
        self.ui.semesters.append(tab)
        if self.english_ui:
            self.ui.courses_tab_widget.insertTab(
                self.ui.courses_tab_widget.count(),
                self.ui.semesters[len(self.ui.semesters) - 1],
                "Semester " + str(self.ui.courses_tab_widget.count() + 1))
        else:
            self.ui.courses_tab_widget.insertTab(
                self.ui.courses_tab_widget.count(),
                self.ui.semesters[len(self.ui.semesters) - 1],
                str(self.ui.courses_tab_widget.count() + 1) + " סמסטר")
        self.update()

    # Function which is being called on clear-row button clicked in the table event
    # the function clears rows content, if the row is clear removes the row
    def clearRow(self, table):
        row = self.find_row(table, self.sender().objectName())
        empty = self.checkIfRowIsEmpty(table, row)
        if empty:
            table.removeRow(row)
            self.update()
            return
        table.cellWidget(row, 0).setCurrentIndex(0)
        for column in range(1, table.columnCount() - 1):
            if table.item(row, column) is not None:
                table.item(row, column).setText("")
                table.item(row, column).setToolTip("")
            if column >= 3:
                table.cellWidget(row, column).setValue(0)
        self.update()

    # Function which finds the row number of signal sender
    def find_row(self, table, senders_id):
        for row in range(0, table.rowCount()):
            clear_button_cell = table.columnCount() - 1
            if table.cellWidget(row, clear_button_cell):
                if table.cellWidget(row, clear_button_cell).objectName() == senders_id:
                    return row
        return table.rowCount()

    # Function which updates the numeric order of semester names in case of removal
    def updateTabNames(self):
        for i in range(self.ui.courses_tab_widget.count()):
            if self.english_ui:
                tab_name = "Semester " + str(i + 1)
            else:
                tab_name = "סמסטר " + str(i + 1)
            self.ui.courses_tab_widget.setTabText(i, tab_name)

    # Function which check if the semester has any input data,
    # if so prompts the user to ensure that he wants to remove the semester
    # and deletes it
    def removeSemester(self, i, force=False):
        empty_semester = True
        if self.english_ui:
            table = self.ui.courses_tab_widget.widget(i).children()[1]
        else:
            table = self.ui.courses_tab_widget.widget(i).children()[7]
        for row in range(table.rowCount()):
            if not self.checkIfRowIsEmpty(table, row):
                empty_semester = False
                break
        if force or empty_semester or (self.english_ui and self.my_close("semester", "Remove non-empty semester?")) or (
                not self.english_ui and self.my_close("semester", "למחוק סמסטר בעל תוכן?")):
            self.ui.courses_tab_widget.removeTab(i)
            if self.ui.courses_tab_widget.count() == 0:
                self.addSemester()
            self.updateTabNames()
            self.update()

    # Function which adds an empty row to the current semester
    def addRow(self, table):
        table.setRowCount(table.rowCount() + 1)
        table.setItem(table.rowCount() - 1, 0, QtWidgets.QTableWidgetItem())
        if self.english_ui:
            table.setCellWidget(table.rowCount() - 1, 0, createComboBox())
        else:
            table.setCellWidget(table.rowCount() - 1, 0, Ui_tab_heb.createComboBox())
        for column in range(1, table.columnCount() - 1):
            item = QtWidgets.QTableWidgetItem()
            item.setTextAlignment(QtCore.Qt.AlignCenter)
            table.setItem(table.rowCount() - 1, column, item)
            if column >= 3:
                if column == 3:
                    spin_box = QtWidgets.QDoubleSpinBox()
                    spin_box.setDecimals(1)
                    spin_box.setSingleStep(0.5)
                else:
                    spin_box = QtWidgets.QSpinBox()
                    spin_box.setSingleStep(1)
                spin_box.setRange(0, 100)
                spin_box.setButtonSymbols(QtWidgets.QAbstractSpinBox.NoButtons)
                spin_box.setAlignment(QtCore.Qt.AlignCenter)
                table.setCellWidget(table.rowCount() - 1, column, spin_box)
                table.cellWidget(table.rowCount() - 1, column).valueChanged.connect(self.update)
        if self.english_ui:
            button = createRemoveLineButton(str(table.rowCount() - 1))
        else:
            button = Ui_tab_heb.createRemoveLineButton(str(table.rowCount() - 1))
        button.clicked.connect(lambda state: self.clearRow(table))
        table.cellWidget(table.rowCount() - 1, 0).currentIndexChanged['int'].connect(self.update)
        table.setCellWidget(table.rowCount() - 1, table.columnCount() - 1, button)

    # Function which deletes the last row from the semester
    # if it is not empty prompts request 
    def removeRow(self, table):
        rows = table.rowCount()
        if rows == 0:
            return
        if self.checkIfRowIsEmpty(table, table.rowCount() - 1):
            table.setRowCount(rows - 1)
        else:
            if self.english_ui:
                answer = self.my_close("row", "Remove non-empty row?")
            else:
                answer = self.my_close("row", "למחוק שורה בעלת תוכן?")
            if answer:
                table.setRowCount(rows - 1)
        self.update()

    # Function which transforms check if there is "do not ask again" flag for removal
    # was checked if so allows removal without dialog, otherwise opens dialog to ensure removal
    def my_close(self, not_show_param, msg):
        if not_show_param == "row" and not self.not_show_remove_course:
            answer = self.warningMsg(not_show_param=not_show_param, msg=msg)
            return answer
        elif not_show_param == "semester" and not self.not_show_remove_semester:
            answer = self.warningMsg(not_show_param=not_show_param, msg=msg)
            return answer
        return True

    # Function which displays error message
    def errorMsg(self, msg):
        if self.english_ui:
            msg_box = QtWidgets.QMessageBox(QtWidgets.QMessageBox.Question,
                                            "Error", msg)
            msg_box.addButton(QtWidgets.QPushButton('Proceed'), QtWidgets.QMessageBox.YesRole)
        else:
            msg_box = QtWidgets.QMessageBox(QtWidgets.QMessageBox.Question,
                                            "שגיאה", msg)
            msg_box.addButton(QtWidgets.QPushButton('המשך'), QtWidgets.QMessageBox.YesRole)
        msg_box.exec()

    # Function which creates a prompt of removal or save
    def warningMsg(self, title="", not_show_param='', msg='ERROR'):
        if self.english_ui:
            if title == "":
                title = "Remove"
            check_box = QtWidgets.QCheckBox("Don't ask again.")
            msg_box = QtWidgets.QMessageBox(self)
            msg_box = QtWidgets.QMessageBox(QtWidgets.QMessageBox.Question, title, msg)
            msg_box.addButton(QtWidgets.QPushButton('Yes'), QtWidgets.QMessageBox.YesRole)
            msg_box.addButton(QtWidgets.QPushButton('No'), QtWidgets.QMessageBox.NoRole)
        else:
            if title == "":
                title = "מחיקה"
            check_box = QtWidgets.QCheckBox("לא להראות שוב")
            msg_box = QtWidgets.QMessageBox(self)
            msg_box = QtWidgets.QMessageBox(QtWidgets.QMessageBox.Question, title, msg)
            msg_box.addButton(QtWidgets.QPushButton('כן'), QtWidgets.QMessageBox.YesRole)
            msg_box.addButton(QtWidgets.QPushButton('לא'), QtWidgets.QMessageBox.NoRole)
        if not_show_param == "semester" or not_show_param == "row":
            msg_box.setCheckBox(check_box)
        reply = msg_box.exec()
        if not_show_param == "semester":
            self.not_show_remove_semester = bool(check_box.isChecked())
        elif not_show_param == "row":
            self.not_show_remove_course = bool(check_box.isChecked())
        if reply == 0:
            return True
        elif reply == 1:
            return False

    # Function which creates a dependencies warning message
    def prerequisitesWarningMessage(self, msg):
        if self.english_ui:
            msg += "Add anyway?"
            msg_box = QtWidgets.QMessageBox(self)
            msg_box = QtWidgets.QMessageBox(QtWidgets.QMessageBox.Question, "Warning", msg)
            msg_box.addButton(QtWidgets.QPushButton('Yes'), QtWidgets.QMessageBox.YesRole)
            msg_box.addButton(QtWidgets.QPushButton('No'), QtWidgets.QMessageBox.NoRole)
        else:
            msg += "להוסיף בכל זאת?"
            msg_box = QtWidgets.QMessageBox(self)
            msg_box = QtWidgets.QMessageBox(QtWidgets.QMessageBox.Question, "התראה", msg)
            msg_box.addButton(QtWidgets.QPushButton('כן'), QtWidgets.QMessageBox.YesRole)
            msg_box.addButton(QtWidgets.QPushButton('לא'), QtWidgets.QMessageBox.NoRole)
        reply = msg_box.exec()
        if reply == 0:
            return True
        elif reply == 1:
            return False



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'progress.ui'
#
# Created by: PyQt5 UI code generator 5.12.1
#
# WARNING! All changes made in this file will be lost!
# Li
#

from PyQt5 import QtCore, QtWidgets


class ProgressWindowForm(object):
    def setupUi(self, Form, language):
        Form.setObjectName("Form")
        Form.resize(320, 180)
        Form.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.gridLayout_2 = QtWidgets.QGridLayout(Form)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.frame = QtWidgets.QFrame(Form)
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.gridLayout = QtWidgets.QGridLayout(self.frame)
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(self.frame)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.progressBar = QtWidgets.QProgressBar(self.frame)
        if not language:
            self.progressBar.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.progressBar.setProperty("value", 24)
        self.progressBar.setAlignment(QtCore.Qt.AlignCenter)
        self.progressBar.setOrientation(QtCore.Qt.Horizontal)
        self.progressBar.setInvertedAppearance(False)
        self.progressBar.setTextDirection(QtWidgets.QProgressBar.TopToBottom)
        self.progressBar.setObjectName("progressBar")
        self.verticalLayout.addWidget(self.progressBar)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.pushButton = QtWidgets.QPushButton(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButton.sizePolicy().hasHeightForWidth())
        self.pushButton.setSizePolicy(sizePolicy)
        self.pushButton.setObjectName("abort-button")
        self.horizontalLayout.addWidget(self.pushButton)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.gridLayout.addLayout(self.verticalLayout, 0, 0, 1, 1)
        self.gridLayout_2.addWidget(self.frame, 0, 0, 1, 1)

        self.retranslateUi(Form, language)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form, language):
        _translate = QtCore.QCoreApplication.translate
        if language:
            Form.setWindowTitle(_translate("Form", "Update"))
            self.pushButton.setText(_translate("Form", "Abort"))
        else:
            self.pushButton.setText(_translate("Form", "ביטול"))
            Form.setWindowTitle(_translate("Form", "עדכון נתונים"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/vov4ik/Projects/DegreePlanner/findDialog.ui'
#
# Created by: PyQt5 UI code generator 5.12.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

from Ui_combo_box import ExtendedComboBox


class Ui_course_search(object):
    def setupUi(self, course_search, english_ui):
        course_search.setObjectName("course_search")
        course_search.resize(498, 443)
        self.gridLayout = QtWidgets.QGridLayout(course_search)
        self.gridLayout.setObjectName("gridLayout")
        self.course_num_label = QtWidgets.QLabel(course_search)
        self.course_num_label.setStyleSheet("font: 12pt \"Noto Sans\";")
        self.course_num_label.setObjectName("course_num_label")
        self.gridLayout.addWidget(self.course_num_label, 0, 0, 1, 1)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.pushButton = QtWidgets.QPushButton(course_search)
        self.pushButton.setMaximumSize(QtCore.QSize(50, 16777215))
        self.pushButton.setText("")
        icon = QtGui.QIcon()
        if english_ui:
            icon.addPixmap(QtGui.QPixmap("images/row-clean.svg").transformed(QtGui.QTransform().rotate(180)),
                           QtGui.QIcon.Normal, QtGui.QIcon.Off)
        else:
            icon.addPixmap(QtGui.QPixmap("images/row-clean.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton.setIcon(icon)
        self.pushButton.setIconSize(QtCore.QSize(32, 32))
        self.pushButton.setObjectName("pushButton")
        self.comboBox = ExtendedComboBox(course_search)
        self.comboBox.setCurrentText("")
        self.comboBox.setMaxVisibleItems(50)
        self.comboBox.setObjectName("comboBox")
        if english_ui:
            self.horizontalLayout.addWidget(self.comboBox)
            self.horizontalLayout.addWidget(self.pushButton)
        else:
            self.horizontalLayout.addWidget(self.pushButton)
            self.horizontalLayout.addWidget(self.comboBox)
            self.comboBox.setLayoutDirection(QtCore.Qt.RightToLeft)
            self.comboBox.lineEdit().setLayoutDirection(QtCore.Qt.RightToLeft)
            self.comboBox.lineEdit().setLayoutDirection(QtCore.Qt.RightToLeft)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.checkBox = QtWidgets.QCheckBox(course_search)
        if not english_ui:
            self.checkBox.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.checkBox.setObjectName("checkBox")
        self.verticalLayout.addWidget(self.checkBox)
        self.course_info_label = QtWidgets.QLabel(course_search)
        self.course_info_label.setStyleSheet("font: 12pt \"Noto Sans\";")
        self.course_info_label.setObjectName("course_info_label")
        self.verticalLayout.addWidget(self.course_info_label)
        self.find_course_in = QtWidgets.QTextEdit(course_search)
        self.find_course_in.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.find_course_in.setLocale(QtCore.QLocale(QtCore.QLocale.Hebrew, QtCore.QLocale.Israel))
        self.find_course_in.setAutoFormatting(QtWidgets.QTextEdit.AutoNone)
        self.find_course_in.setTextInteractionFlags(
            QtCore.Qt.TextSelectableByKeyboard | QtCore.Qt.TextSelectableByMouse)
        self.find_course_in.setObjectName("find_course_in")
        self.verticalLayout.addWidget(self.find_course_in)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.close_search_button = QtWidgets.QPushButton(course_search)
        self.close_search_button.setObjectName("close_search_button")
        self.horizontalLayout_2.addWidget(self.close_search_button)
        self.add_course_button = QtWidgets.QPushButton(course_search)
        self.add_course_button.setObjectName("add_course_button")
        self.horizontalLayout_2.addWidget(self.add_course_button)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.gridLayout.addLayout(self.verticalLayout, 1, 0, 1, 1)

        self.retranslateUi(course_search, english_ui)
        self.pushButton.clearFocus()
        self.pushButton.clicked.connect(self.comboBox.clearEditText)
        QtCore.QMetaObject.connectSlotsByName(course_search)
        course_search.setTabOrder(self.comboBox, self.pushButton)
        course_search.setTabOrder(self.close_search_button, self.find_course_in)

    def retranslateUi(self, course_search, english_ui):
        _translate = QtCore.QCoreApplication.translate
        if english_ui:
            self.course_num_label.setText(_translate("course_search", "Enter course number:"))
            self.course_info_label.setText(_translate("course_search", "Course information:"))
            self.close_search_button.setText(_translate("course_search", "Close"))
            self.add_course_button.setText(_translate("course_search", "Add"))
            self.checkBox.setText(_translate("course_search", "Check prerequisites"))
        else:
            self.course_num_label.setText(_translate("course_search", "הכנס מספר קורס:"))
            self.course_info_label.setText(_translate("course_search", "מידע על הקורס:"))
            self.close_search_button.setText(_translate("course_search", "סגור"))
            self.add_course_button.setText(_translate("course_search", "הוסף"))
            self.checkBox.setText(_translate("course_search", "בדיקת קדמים/צמודים"))
        self.find_course_in.setHtml(_translate("course_search",
                                               "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
                                               "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
                                               "p, li { white-space: pre-wrap; }\n"
                                               "</style></head><body style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
                                               "<p align=\"right\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>"))
